@@ -1,31 +1,32 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from datetime import datetime
+from typing import Any
 
+from flask_sqlalchemy import BaseQuery
 from sqlalchemy.schema import Column
 from sqlalchemy.types import DateTime, Integer
 from whoosh.fields import ID
 
-from ..extensions import db
-from ..util import fqcn
+from abilian.core.extensions import db
+from abilian.core.util import fqcn
+
 
 #: Base Model class.
-Model = db.Model
+class Model(db.Model):
+    __abstract__ = True
+    query: BaseQuery
 
 
 class Info(dict):
-    def __init__(self, **kw):
+    def __init__(self, **kw: Any) -> None:
         for k, v in kw.items():
             self[k] = v
 
-    def copy(self):
+    def copy(self) -> "Info":
         # dict.copy would return an instance of dict
         return self.__class__(**self)
 
-    def __add__(self, other):
+    def __add__(self, other: "Info") -> "Info":
         d = self.copy()
         d.update(other)
         return d
@@ -48,35 +49,33 @@ NOT_EXPORTABLE = Info(exportable=False)
 SYSTEM = Info(editable=False, auditable=False)
 
 
-class IdMixin(object):
+class IdMixin:
     id = Column(Integer, primary_key=True, info=SYSTEM | SEARCHABLE)
 
 
-class Indexable(object):
+class Indexable:
     """Mixin with sensible defaults for indexable objects."""
 
     __indexable__ = True
-    __indexation_args__ = {
-        "index_to": (
-            ("object_key", (("object_key", ID(stored=True, unique=True)),)),
-            ("object_type", (("object_type", ID(stored=True, unique=False)),)),
-        )
-    }
+    __index_to__ = (
+        ("object_key", (("object_key", ID(stored=True, unique=True)),)),
+        ("object_type", (("object_type", ID(stored=True)),)),
+    )
 
     @classmethod
-    def _object_type(cls):
+    def _object_type(cls) -> str:
         return fqcn(cls)
 
     @property
-    def object_type(self):
+    def object_type(self) -> str:
         return self._object_type()
 
     @property
-    def object_key(self):
-        return "{}:{}".format(self.object_type, self.id)
+    def object_key(self) -> str:
+        return f"{self.object_type}:{self.id}"
 
 
-class TimestampedMixin(object):
+class TimestampedMixin:
     #: creation date
     created_at = Column(DateTime, default=datetime.utcnow, info=SYSTEM | SEARCHABLE)
     #: last modification date
