@@ -1,11 +1,8 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import abc
+from typing import Any
 
-import six
+from flask_sqlalchemy import BaseQuery
 from sqlalchemy import CheckConstraint, Column, ForeignKey, Integer, \
     UnicodeText
 from sqlalchemy.ext.declarative import declared_attr
@@ -18,12 +15,11 @@ from abilian.services.security import CREATE, DELETE, WRITE, Anonymous, Owner
 ATTRIBUTE = "__comments__"
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Commentable(object):
+class Commentable(metaclass=abc.ABCMeta):
     pass
 
 
-def register(cls):
+def register(cls: type) -> type:
     """Register an :class:`Entity` as a commentable class.
 
     Can be used as a class decorator:
@@ -41,7 +37,7 @@ def register(cls):
     return cls
 
 
-def is_commentable(obj_or_class):
+def is_commentable(obj_or_class: Any) -> bool:
     """
     :param obj_or_class: a class or instance
     """
@@ -99,6 +95,8 @@ class Comment(Entity):
     #: comment's main content
     body = Column(UnicodeText(), CheckConstraint("trim(body) != ''"), nullable=False)
 
+    query: BaseQuery
+
     @property
     def history(self):
         return self.meta.get("abilian.core.models.comment", {}).get("history", [])
