@@ -1,10 +1,7 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any, Dict, List
 
 from flask import g
-from six import text_type
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import relationship
 from sqlalchemy.schema import Column, ForeignKey
@@ -14,17 +11,15 @@ from .base import AUDITABLE, EDITABLE, SEARCHABLE, SYSTEM
 from .subjects import User
 
 
-class OwnedMixin(object):
-    __indexation_args__ = {
-        "index_to": (
-            ("creator", ("creator",)),
-            ("creator_name", (("creator_name", STORED),)),
-            ("owner", ("owner",)),
-            ("owner_name", (("owner_name", STORED),)),
-        )
-    }
+class OwnedMixin:
+    __index_to__ = (
+        ("creator", ("creator",)),
+        ("creator_name", (("creator_name", STORED),)),
+        ("owner", ("owner",)),
+        ("owner_name", (("owner_name", STORED),)),
+    )
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: List, **kwargs: Dict[str, Any]) -> None:
         try:
             user = g.user
             if not self.creator and not g.user.is_anonymous:
@@ -40,7 +35,7 @@ class OwnedMixin(object):
 
     @declared_attr
     def creator(cls):
-        primary_join = "User.id == {}.creator_id".format(cls.__name__)
+        primary_join = f"User.id == {cls.__name__}.creator_id"
         return relationship(
             User,
             primaryjoin=primary_join,
@@ -50,8 +45,8 @@ class OwnedMixin(object):
         )
 
     @property
-    def creator_name(self):
-        return text_type(self.creator) if self.creator else ""
+    def creator_name(self) -> str:
+        return str(self.creator) if self.creator else ""
 
     @declared_attr
     def owner_id(cls):
@@ -59,7 +54,7 @@ class OwnedMixin(object):
 
     @declared_attr
     def owner(cls):
-        primary_join = "User.id == {}.owner_id".format(cls.__name__)
+        primary_join = f"User.id == {cls.__name__}.owner_id"
         return relationship(
             User,
             primaryjoin=primary_join,
@@ -69,5 +64,5 @@ class OwnedMixin(object):
         )
 
     @property
-    def owner_name(self):
-        return text_type(self.owner) if self.owner else ""
+    def owner_name(self) -> str:
+        return str(self.owner) if self.owner else ""
