@@ -1,4 +1,3 @@
-# coding=utf-8
 """Subject classes (i.e. people, groups, etc.).
 
 See ICOM-ics-v1.0 "Subject Branch".
@@ -6,22 +5,21 @@ See ICOM-ics-v1.0 "Subject Branch".
 TODO: I'm not a big fan of the "subject" name. Could be replaced by something
 else, like "people" or "principal" ?
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import random
 import string
 from abc import ABCMeta, abstractmethod
 from datetime import datetime, timedelta
+from typing import Type, Union
 
 import bcrypt
 import sqlalchemy as sa
 from flask_login import UserMixin
-from six import add_metaclass, python_2_unicode_compatible, text_type
+from flask_sqlalchemy import BaseQuery
 from sqlalchemy.event import listens_for
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import backref, deferred, relationship
-from sqlalchemy.schema import Column, ForeignKey, Table, UniqueConstraint
+from sqlalchemy.orm.mapper import Mapper
+from sqlalchemy.schema import Column, ForeignKey, UniqueConstraint
 from sqlalchemy.types import Boolean, DateTime, Integer, LargeBinary, \
     UnicodeText
 
@@ -30,10 +28,17 @@ from abilian.core.util import fqcn
 
 from .base import SEARCHABLE, SYSTEM, IdMixin, Indexable, TimestampedMixin, db
 
-__all__ = ["User", "Group", "Principal", "ClearPasswordStrategy"]
+__all__ = (
+    "User",
+    "Group",
+    "Principal",
+    "ClearPasswordStrategy",
+    "gen_random_password",
+    "create_root_user",
+)
 
 # Tables for many-to-many relationships
-following = Table(
+following = db.Table(
     "following",
     db.Model.metadata,
     Column("follower_id", Integer, ForeignKey("user.id")),
@@ -41,7 +46,7 @@ following = Table(
     UniqueConstraint("follower_id", "followee_id"),
 )
 
-membership = Table(
+membership = db.Table(
     "membership",
     db.Model.metadata,
     Column(
@@ -58,7 +63,7 @@ membership = Table(
 )
 
 # Should not be needed (?)
-administratorship = Table(
+administratorship = db.Table(
     "administratorship",
     db.Model.metadata,
     Column(
@@ -82,8 +87,7 @@ def gen_random_password(length=15):
     return "".join(rg.choice(_RANDOM_PASSWORD_CHARS) for i in range(length))
 
 
-@add_metaclass(ABCMeta)
-class PasswordStrategy(object):
+class PasswordStrategy(metaclass=ABCMeta):
     @property
     @abstractmethod
     def name(self):
@@ -113,7 +117,7 @@ class ClearPasswordStrategy(PasswordStrategy):
         return user.password == password
 
     def process(self, user, password):
-        if not isinstance(password, text_type):
+        if not isinstance(password, str):
             password = password.decode("utf-8")
         return password
 
@@ -125,23 +129,23 @@ class BcryptPasswordStrategy(PasswordStrategy):
     def name(self):
         return "bcrypt"
 
-    def authenticate(self, user, password):
+    def authenticate(self, user: "User", password: str) -> bool:
         current_passwd = user.password
         # crypt work only on bytes, not str (Unicode)
-        if isinstance(current_passwd, text_type):
+        if isinstance(current_passwd, str):
             current_passwd = current_passwd.encode("utf-8")
-        if isinstance(password, text_type):
+        if isinstance(password, str):
             password = password.encode("utf-8")
 
         return bcrypt.hashpw(password, current_passwd) == current_passwd
 
-    def process(self, user, password):
-        if isinstance(password, text_type):
+    def process(self, user: "User", password: str) -> str:
+        if isinstance(password, str):
             password = password.encode("utf-8")
         return bcrypt.hashpw(password, bcrypt.gensalt()).decode("utf-8")
 
 
-class UserQuery(db.Model.query_class):
+class UserQuery(BaseQuery):
     def get_by_email(self, email):
         return self.filter_by(email=email).one()
 
@@ -149,11 +153,7 @@ class UserQuery(db.Model.query_class):
 class Principal(IdMixin, TimestampedMixin, Indexable):
     """A principal is either a User or a Group."""
 
-    __indexation_args__ = {}
-    __indexation_args__.update(Indexable.__indexation_args__)
-    index_to = __indexation_args__.setdefault("index_to", ())
-    __indexation_args__["index_to"] += (("name", ("name", "name_prefix", "text")),)
-    del index_to
+    __index_to__ = (("name", ("name", "name_prefix", "text")),)
 
     def has_role(self, role, context=None):
         from abilian.services import get_service
@@ -162,7 +162,9 @@ class Principal(IdMixin, TimestampedMixin, Indexable):
         return security.has_role(self, role, context)
 
 
-def set_entity_type(cls):
+def set_entity_type(
+    cls: Union[Type["User"], Type["Group"]]
+) -> Union[Type["User"], Type["Group"]]:
     """Decorator used to set the class' entity_type after the class has been
     declared.
 
@@ -173,7 +175,6 @@ def set_entity_type(cls):
     return cls
 
 
-@python_2_unicode_compatible
 @set_entity_type
 class User(Principal, UserMixin, db.Model):
     __tablename__ = "user"
@@ -183,6 +184,7 @@ class User(Principal, UserMixin, db.Model):
     __password_strategy__ = BcryptPasswordStrategy()
 
     query_class = UserQuery
+    query: UserQuery
 
     # Basic information
     first_name = Column(UnicodeText, info=SEARCHABLE)
@@ -219,13 +221,13 @@ class User(Principal, UserMixin, db.Model):
             self.set_password(password)
             self._password = password
 
-    def authenticate(self, password):
+    def authenticate(self, password: str) -> bool:
         if self.password and self.password != "*":
             return self.__password_strategy__.authenticate(self, password)
         else:
             return False
 
-    def set_password(self, password):
+    def set_password(self, password: str) -> None:
         """Encrypts and sets password."""
         self.password = self.__password_strategy__.process(self, password)
 
@@ -269,20 +271,18 @@ class User(Principal, UserMixin, db.Model):
     # Other properties
     #
     @property
-    def name(self):
-        name = "{first_name} {last_name}".format(
-            first_name=self.first_name or "", last_name=self.last_name or ""
-        )
+    def name(self) -> str:
+        name = f"{(self.first_name or '')} {(self.last_name or '')}"
         return name.strip() or "Unknown"
 
     @property
     def short_name(self):
         first_name = self.first_name or ""
         last_name = self.last_name[0:1] + "." if self.last_name else ""
-        name = "{} {}".format(first_name, last_name)
+        name = f"{first_name} {last_name}"
         return name.strip() or "Unknown"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
     def __repr__(self):
@@ -297,7 +297,7 @@ class User(Principal, UserMixin, db.Model):
 
 
 @listens_for(User, "mapper_configured", propagate=True)
-def _add_user_indexes(mapper, class_):
+def _add_user_indexes(mapper: Mapper, class_: type) -> None:
     # this is a functional index (indexes on a function result), we cannot define
     # it in __table_args__.
     #
@@ -342,3 +342,14 @@ class Group(Principal, db.Model):
             .group_by(membership.c.group_id)
             .label("members_count")
         )
+
+
+def create_root_user() -> User:
+    user = User.query.get(0)
+    if user is None:
+        user = User(
+            id=0, last_name="SYSTEM", email="system@example.com", can_login=False
+        )
+        db.session.add(user)
+        db.session.commit()
+    return user
