@@ -1,15 +1,12 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from datetime import datetime, timedelta
 
+from flask import Flask
 from pytest import raises
 
 from abilian.core.entities import Entity
-
-from ..comment import Comment, is_commentable, register
+from abilian.core.models.comment import Comment, is_commentable, register
+from abilian.core.sqlalchemy import SQLAlchemy
 
 
 @register
@@ -17,7 +14,7 @@ class CommentableContent(Entity):
     pass
 
 
-def test_commentable_interface():
+def test_commentable_interface() -> None:
     assert is_commentable(CommentableContent)
 
     instance = CommentableContent(name="test instance")
@@ -29,15 +26,15 @@ def test_commentable_interface():
     assert not is_commentable(object())
 
 
-def test_cannot_register_non_entities():
-    class Dummy(object):
+def test_cannot_register_non_entities() -> None:
+    class Dummy:
         pass
 
     with raises(ValueError):
         register(Dummy)
 
 
-def test_default_ordering(app, db):
+def test_default_ordering(app: Flask, db: SQLAlchemy) -> None:
     commentable = CommentableContent(name="commentable objet")
     db.session.add(commentable)
 
