@@ -1,13 +1,10 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from datetime import datetime
 
 from pytest import fixture
 from sqlalchemy import create_engine
 from sqlalchemy.orm import sessionmaker
+from sqlalchemy.orm.session import Session
 
 from abilian.core.entities import Entity
 from abilian.core.models.base import AUDITABLE, NOT_SEARCHABLE, SEARCHABLE, \
@@ -18,7 +15,7 @@ from .dummy import DummyContact
 
 
 @fixture
-def session():
+def session() -> Session:
     engine = create_engine("sqlite:///:memory:", echo=False)
     session_class = sessionmaker(bind=engine)
     session = session_class()
@@ -30,7 +27,7 @@ def session():
     return session
 
 
-def test():
+def test() -> None:
     contact = DummyContact(first_name="John")
     assert contact.creator is None
     assert contact.owner is None
@@ -40,7 +37,7 @@ def test():
     contact.creator = user
 
 
-def test_auto_slug_property(session):
+def test_auto_slug_property(session: Session) -> None:
     obj = DummyContact(name="a b c")
     session.add(obj)
     session.flush()
@@ -64,7 +61,7 @@ def test_auto_slug_property(session):
     assert "\u002d" in slug
 
 
-def test_updated_at(session):
+def test_updated_at(session: Session) -> None:
     contact = DummyContact()
     session.add(contact)
     session.commit()
@@ -77,7 +74,7 @@ def test_updated_at(session):
     assert contact.updated_at > updated
 
 
-def test_auto_slug(session):
+def test_auto_slug(session: Session) -> None:
     contact1 = DummyContact(name="Pacôme Hégésippe Adélard Ladislas")
     session.add(contact1)
     session.flush()
@@ -87,7 +84,7 @@ def test_auto_slug(session):
     contact2 = DummyContact()
     session.add(contact2)
     session.flush()
-    expected = "dummycontact-{}".format(contact2.id)
+    expected = f"dummycontact-{contact2.id}"
     assert contact2.slug == expected
 
     # test numbering if slug already exists:
@@ -97,7 +94,7 @@ def test_auto_slug(session):
     assert contact3.slug == "pacome-hegesippe-adelard-ladislas-1"
 
 
-def test_polymorphic_update_timestamp(session):
+def test_polymorphic_update_timestamp(session: Session) -> None:
     contact = DummyContact(name="Pacôme Hégésippe Adélard Ladislas")
     session.add(contact)
     session.flush()
@@ -109,7 +106,7 @@ def test_polymorphic_update_timestamp(session):
     assert contact.updated_at > updated_at
 
 
-def test_meta(session):
+def test_meta(session: Session) -> None:
     e = DummyContact(name="test")
     e.meta["key"] = "value"
     e.meta["number"] = 42
@@ -123,7 +120,7 @@ def test_meta(session):
     assert e.meta["number"] == 42
 
 
-def test_entity_type():
+def test_entity_type() -> None:
     class MyType(Entity):
         pass
 
@@ -144,7 +141,7 @@ def test_entity_type():
     assert OtherBase._object_type() == "some.module.OtherBase"
 
     # test when ENTITY_TYPE_BASE is in ancestors
-    class Base(object):
+    class Base:
         ENTITY_TYPE_BASE = "from.ancestor"
 
     class InheritedBase(Base, Entity):
@@ -154,7 +151,7 @@ def test_entity_type():
     assert InheritedBase._object_type() == "from.ancestor.InheritedBase"
 
 
-def test_info():
+def test_info() -> None:
     info = SEARCHABLE
     assert info["searchable"]
 
