@@ -1,8 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from abilian.core.singleton import UniqueName
 
 
@@ -14,20 +10,20 @@ class NS2(UniqueName):
     pass
 
 
-def test_singleton():
+def test_singleton() -> None:
     val = NS1("val")
     other_val = NS1("val")
     assert val is other_val
     assert id(val) == id(other_val)
 
 
-def test_equality():
+def test_equality() -> None:
     val = NS1("val")
     assert val == "val"
     assert val == "val"
 
 
-def test_namespaces():
+def test_namespaces() -> None:
     ns1_val = NS1("val")
     ns2_val = NS2("val")
     assert ns1_val is not ns2_val
