@@ -1,8 +1,4 @@
-# coding=utf-8
 """Various tools that don't belong some place specific."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import functools
 import hashlib
 import logging
@@ -11,16 +7,15 @@ import time
 import unicodedata
 from datetime import datetime
 from math import ceil
+from typing import Any, Union
 
 import pytz
-import six
 from babel.dates import LOCALTZ
 from flask import request
-from six import text_type
 from werkzeug.local import LocalProxy
 
 
-def unwrap(obj):
+def unwrap(obj: Any):
     """Unwrap obj from werkzeug.local.LocalProxy if needed.
 
     This is required if one want to test `isinstance(obj, SomeClass)`.
@@ -34,28 +29,30 @@ def unwrap(obj):
 noproxy = unwrap
 
 
-def fqcn(cls):
+def fqcn(cls: type) -> str:
     """Fully Qualified Class Name."""
-    return text_type(cls.__module__ + "." + cls.__name__)
+    return str(cls.__module__ + "." + cls.__name__)
 
 
-def friendly_fqcn(cls_name):
+def friendly_fqcn(cls_or_cls_name: Union[type, str]) -> str:
     """Friendly name of fully qualified class name.
 
-    :param cls_name: a string or a class
+    :param cls_or_cls_name: a string or a class
     """
-    if isinstance(cls_name, type):
-        cls_name = fqcn(cls_name)
+    if isinstance(cls_or_cls_name, type):
+        cls_name = fqcn(cls_or_cls_name)
+    else:
+        cls_name = cls_or_cls_name
 
     return cls_name.rsplit(".", 1)[-1]
 
 
-def utcnow():
+def utcnow() -> datetime:
     """Return a new aware datetime with current date and time, in UTC TZ."""
     return datetime.now(pytz.utc)
 
 
-def local_dt(dt):
+def local_dt(dt: datetime) -> datetime:
     """Return an aware datetime in system timezone, from a naive or aware
     datetime.
 
@@ -66,7 +63,7 @@ def local_dt(dt):
     return LOCALTZ.normalize(dt.astimezone(LOCALTZ))
 
 
-def utc_dt(dt):
+def utc_dt(dt: datetime) -> datetime:
     """Set UTC timezone on a datetime object.
 
     A naive datetime is assumed to be in UTC TZ.
@@ -90,7 +87,7 @@ def get_params(names):
     return params
 
 
-class timer(object):
+class timer:
     """Decorator that mesures the time it takes to run a function."""
 
     def __init__(self, f):
@@ -101,12 +98,12 @@ class timer(object):
         self.__start = time.time()
         result = self.__f(*args, **kwargs)
         value = time.time() - self.__start
-        self.log.info("elapsed time: {:.2f}ms".format(value * 1000))
+        self.log.info(f"elapsed time: {(value * 1000):.2f}ms")
         return result
 
 
 # From http://wiki.python.org/moin/PythonDecoratorLibrary#Memoize
-class memoized(object):
+class memoized:
     """Decorator that caches a function's return value each time it is called.
 
     If called later with the same arguments, the cached value is
@@ -136,7 +133,7 @@ class memoized(object):
 
 
 # From http://flask.pocoo.org/snippets/44/
-class Pagination(object):
+class Pagination:
     def __init__(self, page, per_page, total_count):
         self.page = page
         self.per_page = per_page
@@ -182,20 +179,18 @@ _NOT_WORD_RE = re.compile(r"[^\w\s]+", flags=re.UNICODE)
 
 def slugify(value, separator="-"):
     """Slugify an Unicode string, to make it URL friendly."""
-    if six.PY2:
-        value = text_type(value)
-    if not isinstance(value, text_type):
+    if not isinstance(value, str):
         raise ValueError("value must be a Unicode string")
     value = _NOT_WORD_RE.sub(" ", value)
     value = unicodedata.normalize("NFKD", value)
     value = value.encode("ascii", "ignore")
     value = value.decode("ascii")
     value = value.strip().lower()
-    value = re.sub(r"[{}_\s]+".format(separator), separator, value)
+    value = re.sub(fr"[{separator}_\s]+", separator, value)
     return value
 
 
-class BasePresenter(object):
+class BasePresenter:
     """A presenter wraps a model an adds specific (often, web-centric)
     accessors.
 
@@ -220,22 +215,23 @@ class BasePresenter(object):
         return [cls(model) for model in models]
 
 
-def encode_string(string):
+def encode_string(string_or_bytes: Union[str, bytes]) -> bytes:
     """Encode a string to bytes, if it isn't already.
 
-    :param string: The string to encode
+    :param string_or_bytes: The string to encode.
+        Does nothing if it's already bytes.
     """
     # from pysecurity
 
-    if isinstance(string, text_type):
-        return string.encode("utf-8")
+    if isinstance(string_or_bytes, str):
+        return string_or_bytes.encode()
     else:
-        return string
+        return string_or_bytes
 
 
-def md5(data):
+def md5(data: Union[str, bytes, None]):
     """md5 function, as in flask-security."""
     # flask-security is not needed anywhere else and as of 1.7.5 has
     # strong dependency requirements that prevent us from upgrading a
     # few packages (flask-login 0.4).
-    return hashlib.md5(encode_string(data)).hexdigest()
+    return hashlib.md5(encode_string(data or "")).hexdigest()
