@@ -1,16 +1,16 @@
-# coding=utf-8
 """Modules that provide services.
 
 They are implemented as Flask extensions (see:
 http://flask.pocoo.org/docs/extensiondev/ )
 """
-from __future__ import absolute_import, division, print_function
-
 from flask import current_app
 
-# This one first
+# This one must be imported first
 from .base import Service, ServiceState
 
+# Don't remove (used to force import order)
+assert Service, ServiceState
+
 from .activity import ActivityService
 from .antivirus import service as antivirus
 from .audit import audit_service
@@ -29,5 +29,5 @@ activity_service = ActivityService()
 settings_service = SettingsService()
 
 
-def get_service(service):
+def get_service(service: str) -> Service:
     return current_app.services[service]
