@@ -1,4 +1,3 @@
-# coding=utf-8
 """Activity Service.
 
 See: http://activitystrea.ms/specs/json/1.0/
@@ -7,13 +6,10 @@ See: http://stackoverflow.com/questions/1443960/
 
 TODO: Look wether other attributes from the spec need to be implemented.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 from datetime import datetime
+from typing import Callable
 
-from six import text_type
 from sqlalchemy.orm import relationship
 from sqlalchemy.schema import Column, ForeignKey
 from sqlalchemy.types import DateTime, Integer, String, Text
@@ -26,7 +22,7 @@ __all__ = ["ActivityEntry"]
 logger = logging.getLogger(__name__)
 
 
-def _default_from(column):
+def _default_from(column: str) -> Callable:
     """Helper for default and onupdates parameters in a Column definitions.
 
     Returns a `context-sensitive default function
@@ -66,8 +62,8 @@ class ActivityEntry(db.Model):
         return tpl.format(
             self.__class__.__module__,
             self.id,
-            repr(text_type(self.actor)),
+            repr(str(self.actor)),
             repr(self.verb),
-            repr(text_type(self.object)),
-            repr(text_type(self.target)),
+            repr(str(self.object)),
+            repr(str(self.target)),
         )
