@@ -1,4 +1,3 @@
-# coding=utf-8
 """audit Service: logs modifications to audited objects.
 
 TODO: In the future, we may decide to:
@@ -6,21 +5,18 @@ TODO: In the future, we may decide to:
 - Make Models that have the __auditable__ property (set to True) auditable.
 - Make Entities that have the __auditable__ property set to False not auditable.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 from datetime import datetime
 from inspect import isclass
-from typing import Text
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set
 
 import sqlalchemy as sa
 from flask import current_app, g
-from six import text_type
+from flask_sqlalchemy import Model
 from sqlalchemy import event, extract
-from sqlalchemy.orm import Query
-from sqlalchemy.orm.attributes import NEVER_SET
-from sqlalchemy.orm.session import Session
+from sqlalchemy.orm import Query, Session
+from sqlalchemy.orm.attributes import NEVER_SET, InstrumentedAttribute
+from sqlalchemy.orm.unitofwork import UOWTransaction
 
 from abilian.core.entities import Entity
 from abilian.core.extensions import db
@@ -28,24 +24,30 @@ from abilian.services import Service, ServiceState
 
 from .models import CREATION, DELETION, RELATED, UPDATE, AuditEntry, Changes
 
+if TYPE_CHECKING:
+    from abilian.app import Application
+
 log = logging.getLogger(__name__)
 
 
-class AuditableMeta(object):
-    name = None
-    id_attr = None
-    related = None
-    backref_attr = None
-    audited_attrs = None
-    collection_attrs = None
-    enduser_ids = None
+class AuditableMeta:
+    backref_attr: Optional[str] = None
+    audited_attrs: Set[InstrumentedAttribute]
+    collection_attrs: Set[InstrumentedAttribute]
+    enduser_ids: List[str]
 
-    def __init__(self, name=None, id_attr=None, related=False):
+    def __init__(
+        self,
+        name: Optional[str] = None,
+        id_attr: Optional[str] = None,
+        related: bool = False,
+    ) -> None:
         self.name = name
         self.id_attr = id_attr
         self.related = related
         self.audited_attrs = set()
         self.collection_attrs = set()
+        self.enduser_ids = []
 
 
 class AuditServiceState(ServiceState):
@@ -57,10 +59,10 @@ class AuditServiceState(ServiceState):
     # of audit entries
     creating_entries = False
 
-    def __init__(self, *args, **kwargs):
-        ServiceState.__init__(self, *args, **kwargs)
-        self.all_model_classes = set()
-        self.model_class_names = {}
+    def __init__(self, service: "AuditService", *args: Any, **kwargs: Any) -> None:
+        super().__init__(service, *args, **kwargs)
+        self.all_model_classes: Set[type] = set()
+        self.model_class_names: Dict[str, type] = {}
 
 
 class AuditService(Service):
@@ -69,19 +71,19 @@ class AuditService(Service):
 
     _listening = False
 
-    def init_app(self, app):
-        Service.init_app(self, app)
+    def init_app(self, app: "Application") -> None:
+        super().init_app(app)
 
         if not self._listening:
             event.listen(Session, "after_flush", self.create_audit_entries)
             self._listening = True
 
-    def start(self, **kw):
-        Service.start(self)
+    def start(self, ignore_state: bool = False) -> None:
+        super().start(ignore_state)
         self.register_classes()
 
     @staticmethod
-    def is_auditable(model_or_class):
+    def is_auditable(model_or_class: Any) -> bool:
         if hasattr(model_or_class, "__auditable_entity__"):
             return True
 
@@ -90,7 +92,7 @@ class AuditService(Service):
         else:
             return isinstance(model_or_class, Entity)
 
-    def register_classes(self):
+    def register_classes(self) -> None:
         state = self.app_state
         BaseModel = db.Model
         all_models = (
@@ -101,7 +103,9 @@ class AuditService(Service):
         for cls in all_models:
             self.register_class(cls, app_state=state)
 
-    def register_class(self, entity_class, app_state=None):
+    def register_class(
+        self, entity_class: Any, app_state: Optional[AuditServiceState] = None
+    ) -> None:
         if not hasattr(entity_class, "__table__"):
             return
 
@@ -133,7 +137,7 @@ class AuditService(Service):
             event.listen(attr, "append", self.collection_append, active_history=True)
             event.listen(attr, "remove", self.collection_remove, active_history=True)
 
-    def setup_auditable_entity(self, entity_class):
+    def setup_auditable_entity(self, entity_class: Any) -> None:
         meta = AuditableMeta(entity_class.__name__, "id")
         entity_class.__auditable__ = meta
 
@@ -180,13 +184,16 @@ class AuditService(Service):
         meta.backref_attr = backref_attr
         meta.enduser_ids = [attr_name.split(".") for attr_name in enduser_ids]
 
-    def _get_changes_for(self, entity):
+    def _get_changes_for(self, entity: Model) -> Changes:
         changes = getattr(entity, "__changes__", None)
         if not changes:
             changes = entity.__changes__ = Changes()
         return changes
 
-    def set_attribute(self, entity, new_value, old_value, initiator):
+    def set_attribute(
+        self, entity: Model, new_value: Any, old_value: Any, initiator: Any
+    ) -> None:
+
         attr_name = initiator.key
         if old_value == new_value:
             return
@@ -209,15 +216,17 @@ class AuditService(Service):
         new_value = format_large_value(new_value)
         changes.set_column_changes(attr_name, old_value, new_value)
 
-    def collection_append(self, entity, value, initiator):
+    def collection_append(self, entity: Entity, value: Any, initiator: Any) -> None:
         changes = self._get_changes_for(entity)
         changes.collection_append(initiator.key, value)
 
-    def collection_remove(self, entity, value, initiator):
+    def collection_remove(self, entity: Entity, value: Any, initiator: Any):
         changes = self._get_changes_for(entity)
         changes.collection_remove(initiator.key, value)
 
-    def create_audit_entries(self, session, flush_context):
+    def create_audit_entries(
+        self, session: Session, flush_context: UOWTransaction
+    ) -> None:
         if not self.running or self.app_state.creating_entries:
             return
 
@@ -237,10 +246,8 @@ class AuditService(Service):
                         entry = self.log(session, model, op)
                         if entry:
                             entries.append(entry)
-                    except BaseException:
-                        if current_app.config.get("DEBUG") or current_app.config.get(
-                            "TESTING"
-                        ):
+                    except Exception:
+                        if current_app.debug or current_app.testing:
                             raise
                         log.error("Exception during entry creation", exc_info=True)
 
@@ -248,14 +255,14 @@ class AuditService(Service):
         finally:
             self.app_state.creating_entries = False
 
-    def log(self, session, model, op_type):
+    def log(self, session: Session, model: Any, op_type: int) -> Optional[AuditEntry]:
         if not self.is_auditable(model):
-            return
+            return None
 
         entity = model
         try:
             user_id = g.user.id
-        except BaseException:
+        except Exception:
             user_id = 0
 
         meta = model.__auditable__
@@ -265,7 +272,7 @@ class AuditService(Service):
             for attr in meta.related:
                 entity = getattr(entity, attr)
                 if entity is None:
-                    return
+                    return None
 
         entry = AuditEntry(type=op_type, user_id=user_id)
         if op_type != DELETION:
@@ -297,7 +304,7 @@ class AuditService(Service):
         elif op == UPDATE:
             changes = getattr(model, "__changes__", changes)
             if not changes:
-                return
+                return None
 
         if hasattr(model, "__changes__"):
             del model.__changes__
@@ -308,9 +315,9 @@ class AuditService(Service):
                 item = model
                 for attr in path:
                     item = getattr(item, attr)
-                enduser_ids.append(text_type(item))
+                enduser_ids.append(str(item))
 
-            related_name = "{} {}".format(meta.backref_attr, " ".join(enduser_ids))
+            related_name = f"{meta.backref_attr} {' '.join(enduser_ids)}"
             related_changes = changes
             log.debug("related changes: %s", repr(related_changes))
             changes = Changes()
@@ -333,7 +340,7 @@ class AuditService(Service):
 audit_service = AuditService()
 
 
-def format_large_value(value):
+def format_large_value(value: Any) -> Any:
     try:
         if len(value) > 1000:
             return "<<large value>>"
@@ -344,9 +351,13 @@ def format_large_value(value):
 
 
 def get_model_changes(
-    entity_type, year=None, month=None, day=None, hour=None, since=None
-):
-    # type: (Text, int, int, int, int, datetime) -> Query
+    entity_type: str,
+    year: Optional[int] = None,
+    month: Optional[int] = None,
+    day: Optional[int] = None,
+    hour: Optional[int] = None,
+    since: Optional[datetime] = None,
+) -> Query:
     """Get models modified at the given date with the Audit service.
 
     :param entity_type: string like "extranet_medicen.apps.crm.models.Compte".
