@@ -1,48 +1,51 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
+import typing
 from datetime import datetime, timedelta
+from typing import Any, Callable, Dict, List, Optional, Union
 
-from flask import current_app, g, redirect, request, url_for
+from flask import Flask, current_app, g, redirect, request, url_for
 from flask_babel import lazy_gettext as _l
 from flask_login import current_user, login_user, user_logged_in, \
     user_logged_out
 from werkzeug.exceptions import Forbidden
+from werkzeug.wrappers import Response
 
 from abilian.core.extensions import db, login_manager
 from abilian.core.models.subjects import User
 from abilian.core.signals import user_loaded
 from abilian.core.util import unwrap
 from abilian.services import Service, ServiceState
+from abilian.services.security.service import SecurityService
 from abilian.web.action import DynamicIcon, actions
 from abilian.web.nav import NavGroup, NavItem
 
 from .models import LoginSession
 from .views import login as login_views
 
+if typing.TYPE_CHECKING:
+    from abilian.app import Application
+
 __all__ = ["AuthService", "user_menu"]
 
 logger = logging.getLogger(__name__)
 
 
-def is_anonymous(context):
+def is_anonymous(context: Dict[str, bool]) -> bool:
     return current_user.is_anonymous
 
 
-def is_authenticated(context):
+def is_authenticated(context: Dict[str, bool]) -> bool:
     return not is_anonymous(context)
 
 
-def _user_photo_endpoint():
+def _user_photo_endpoint() -> str:
     from abilian.web.views import images  # lazy import: avoid circular import
 
     return images.user_url_args(current_user, 16)[0]
 
 
-def _user_photo_icon_args(icon, url_args):
+def _user_photo_icon_args(icon: DynamicIcon, url_args: Dict) -> Dict[str, Any]:
     from abilian.web.views import images  # lazy import avoid circular import
 
     return images.user_url_args(current_user, max(icon.width, icon.height))[1]
@@ -87,15 +90,15 @@ _ACTIONS = (
 class AuthServiceState(ServiceState):
     """State class for :class:`AuthService`"""
 
-    def __init__(self, *args, **kwargs):
-        super(AuthServiceState, self).__init__(*args, **kwargs)
-        self.bp_access_controllers = {None: []}
-        self.endpoint_access_controllers = {}
+    def __init__(self, service: "AuthService", *args: Any, **kwargs: Any) -> None:
+        super().__init__(service, *args, **kwargs)
+        self.bp_access_controllers: Dict[Optional[str], List[Callable]] = {None: []}
+        self.endpoint_access_controllers: Dict[str, List[Callable]] = {}
 
-    def add_bp_access_controller(self, blueprint, func):
+    def add_bp_access_controller(self, blueprint: str, func: Callable) -> None:
         self.bp_access_controllers.setdefault(blueprint, []).append(func)
 
-    def add_endpoint_access_controller(self, endpoint, func):
+    def add_endpoint_access_controller(self, endpoint: str, func: Callable) -> None:
         self.endpoint_access_controllers.setdefault(endpoint, []).append(func)
 
 
@@ -103,11 +106,11 @@ class AuthService(Service):
     name = "auth"
     AppStateClass = AuthServiceState
 
-    def init_app(self, app):
+    def init_app(self, app: Flask) -> None:
         login_manager.init_app(app)
         login_manager.login_view = "login.login_form"
 
-        Service.init_app(self, app)
+        super().init_app(app)
 
         self.login_url_prefix = app.config.get("LOGIN_URL", "/user")
 
@@ -122,7 +125,7 @@ class AuthService(Service):
             actions.register(*_ACTIONS)
 
     @login_manager.user_loader
-    def load_user(user_id):
+    def load_user(user_id: str) -> Optional[User]:
         try:
             user = User.query.get(user_id)
 
@@ -130,7 +133,7 @@ class AuthService(Service):
                 # if a user is edited and should not have access any more, this
                 # will ensure they cannot continue if he had an active session
                 return None
-        except BaseException:
+        except Exception:
             logger.warning("Error during login.", exc_info=True)
             session = db.session()
             if not session.is_active:
@@ -145,7 +148,7 @@ class AuthService(Service):
         user_loaded.send(app, user=user)
         return user
 
-    def user_logged_in(self, app, user):
+    def user_logged_in(self, app: "Application", user: User) -> None:
         # `g.user` is used as `current_user`, but `current_user` is actually looking
         # for `request.user` whereas `g` is on app local stack.
         #
@@ -158,25 +161,23 @@ class AuthService(Service):
         g.is_manager = (
             user
             and not is_anonymous
-            and (
-                (security.has_role(user, "admin") or security.has_role(user, "manager"))
-            )
+            and (security.has_role(user, "admin") or security.has_role(user, "manager"))
         )
 
-    def user_logged_out(self, app, user):
+    def user_logged_out(self, app: "Application", user: User) -> None:
         if hasattr(g, "user"):
             del g.user
             del g.logged_user
             del g.is_manager
 
-    def redirect_to_login(self, next_url=True):
+    def redirect_to_login(self, next_url=True) -> Response:
         kw = {}
         if next_url is not False:
             kw["next"] = request.url if next_url is True else next_url
 
         return redirect(url_for(login_manager.login_view, **kw))
 
-    def do_access_control(self):
+    def do_access_control(self) -> Union[None, Response]:
         """`before_request` handler to check if user should be redirected to
         login page."""
         from abilian.services import get_service
@@ -185,21 +186,21 @@ class AuthService(Service):
             # Special case for tests
             user = User.query.get(0)
             login_user(user, force=True)
-            return
+            return None
 
         state = self.app_state
         user = unwrap(current_user)
 
         # Another special case for tests
         if current_app.testing and getattr(user, "is_admin", False):
-            return
+            return None
 
-        security = get_service("security")
+        security: SecurityService = get_service("security")
         user_roles = frozenset(security.get_roles(user))
         endpoint = request.endpoint
         blueprint = request.blueprint
 
-        access_controllers = []
+        access_controllers: List[Callable] = []
         access_controllers.extend(state.bp_access_controllers.get(None, []))
 
         if blueprint and blueprint in state.bp_access_controllers:
@@ -213,7 +214,7 @@ class AuthService(Service):
             if verdict is None:
                 continue
             elif verdict is True:
-                return
+                return None
             else:
                 if user.is_anonymous:
                     return self.redirect_to_login()
@@ -223,7 +224,9 @@ class AuthService(Service):
         if current_app.config.get("PRIVATE_SITE") and user.is_anonymous:
             return self.redirect_to_login()
 
-    def update_user_session_data(self):
+        return None
+
+    def update_user_session_data(self) -> None:
         user = current_user
         if current_user.is_anonymous:
             return
@@ -239,7 +242,7 @@ class AuthService(Service):
         refresh_login_session(user)
 
 
-def refresh_login_session(user):
+def refresh_login_session(user: User) -> None:
     now = datetime.utcnow()
     session = LoginSession.query.get_active_for(user)
     if not session:
