@@ -1,23 +1,23 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import json
 from functools import partial
 
 from flask import request, url_for
+from flask.ctx import AppContext
+from flask.testing import FlaskClient
+from sqlalchemy.orm import Session
 
+from abilian.app import Application
 from abilian.core.models.subjects import User
 from abilian.services.auth import views
 
 
-def test_get_redirect_target(app, app_context):
+def test_get_redirect_target(app: Application, app_context: AppContext) -> None:
     get_redirect_target = views.get_redirect_target
     form_url = partial(url_for, "login.login_form")
 
     with app.test_request_context(form_url()):
-        assert get_redirect_target() is None
+        assert get_redirect_target() == ""
         url_root = request.url_root[:-1]
 
     with app.test_request_context(form_url(next="/")):
@@ -33,11 +33,11 @@ def test_get_redirect_target(app, app_context):
     # -> redirect(next = forgot password form) -> ...
     referrer = url_root + url_for("login.forgotten_pw")
     with app.test_request_context(form_url(), headers=[("Referer", referrer)]):
-        assert get_redirect_target() is None
+        assert get_redirect_target() == ""
 
     # test open redirect is forbidden
     with app.test_request_context(form_url(next="http://google.com/test")):
-        assert get_redirect_target() is None
+        assert get_redirect_target() == ""
 
     # open redirect through malicious construct and browser not checking
     # Location
@@ -45,7 +45,7 @@ def test_get_redirect_target(app, app_context):
         assert get_redirect_target() == url_root + "///google.com"
 
 
-def test_login_post(session, client):
+def test_login_post(session: Session, client: FlaskClient) -> None:
     kwargs = {"email": "User@domain.tld", "password": "azerty", "can_login": True}
     user = User(**kwargs)
     session.add(user)
@@ -67,7 +67,7 @@ def test_login_post(session, client):
     assert response.status_code == 401
 
 
-def test_api_post(session, client):
+def test_api_post(session: Session, client: FlaskClient) -> None:
     kwargs = {"email": "User@domain.tld", "password": "azerty", "can_login": True}
     user = User(**kwargs)
     session.add(user)
@@ -88,7 +88,7 @@ def test_api_post(session, client):
     assert response.status_code == 200
 
 
-def test_forgotten_pw(app, session, client):
+def test_forgotten_pw(app: Application, session: Session, client: FlaskClient) -> None:
     mail = app.extensions["mail"]
     kwargs = {"email": "User@domain.tld", "password": "azerty", "can_login": True}
     user = User(**kwargs)
