@@ -1,24 +1,12 @@
-# coding=utf-8
-
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 import os
-import subprocess
 from contextlib import contextmanager
+from pathlib import Path
 from tempfile import mkstemp
-
-try:
-    from six.moves.xmlrpc_client import ServerProxy
-except ImportError:
-    ServerProxy = None
+from typing import Iterator, Optional
 
 logger = logging.getLogger(__name__)
 
-# For some reason, twill includes a broken version of subprocess.
-assert "twill" not in subprocess.__file__
-
 # Hack for Mac OS + homebrew
 os.environ["PATH"] += ":/usr/local/bin"
 
@@ -26,7 +14,7 @@ TMP_DIR = "tmp"
 CACHE_DIR = "cache"
 
 
-def get_tmp_dir():
+def get_tmp_dir() -> Path:
     from . import converter
 
     return converter.tmp_dir
@@ -34,19 +22,26 @@ def get_tmp_dir():
 
 # Utils
 @contextmanager
-def make_temp_file(blob=None, prefix="tmp", suffix="", tmp_dir=None):
+def make_temp_file(
+    blob: Optional[bytes] = None,
+    prefix: str = "tmp",
+    suffix: str = "",
+    tmp_dir: Optional[Path] = None,
+) -> Iterator[str]:
+
     if tmp_dir is None:
         tmp_dir = get_tmp_dir()
 
     fd, filename = mkstemp(dir=str(tmp_dir), prefix=prefix, suffix=suffix)
     if blob is not None:
-        fd = os.fdopen(fd, "wb")
-        fd.write(blob)
-        fd.close()
+        io = os.fdopen(fd, "wb")
+        io.write(blob)
+        io.close()
     else:
         os.close(fd)
 
     yield filename
+
     try:
         os.remove(filename)
     except OSError:
