@@ -1,17 +1,16 @@
-# coding=utf-8
 """Objects to schema adapters."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 from abc import ABCMeta, abstractmethod
 from operator import attrgetter
+from typing import Any, Dict, Optional, Tuple, Type, Union
 
 import sqlalchemy as sa
-from six import add_metaclass, string_types
-from whoosh.fields import TEXT
+from sqlalchemy.orm.session import Session
+from whoosh.fields import ID, TEXT, Schema
 
+from abilian.core.entities import Entity
 from abilian.core.extensions import db
+from abilian.core.models import Model
 
 from .schema import accent_folder
 
@@ -20,20 +19,18 @@ __all__ = ["SchemaAdapter", "SAAdapter"]
 logger = logging.getLogger(__name__)
 
 
-@add_metaclass(ABCMeta)
-class SchemaAdapter(object):
+class SchemaAdapter(metaclass=ABCMeta):
     """Abstract base class for objects to schema adapter.
 
     The purpose of adapters is that given an object they return kwargs
     for document.
     """
 
-    def __init__(self, Model, schema):
+    def __init__(self, model_class, schema):
         """
-        :param:Model: class of objects instances to be adapted
+        :param:model_class: class of objects instances to be adapted
         :param:schema: :class:`whoosh.fields.Schema` instance
         """
-        pass
 
     @staticmethod
     def can_adapt(obj_cls):
@@ -45,6 +42,8 @@ class SchemaAdapter(object):
         """Returns an object instance given its identifier and optional data
         kwargs.
 
+        :param pk: the primary key to retrieve the object
+
         :returns: None if not object instance can be returned
         """
         raise NotImplementedError
@@ -78,34 +77,44 @@ class SAAdapter(SchemaAdapter):
     * define new fields on schema
     """
 
-    @staticmethod
-    def can_adapt(obj_cls):
-        return issubclass(obj_cls, db.Model)
-
-    def __init__(self, Model, schema):
+    def __init__(self, model_class: Type[Model], schema: Schema) -> None:
         """
-        :param:Model: a sqlalchemy model class
+        :param:model_class: a sqlalchemy model class
         :param:schema: :class:`whoosh.fields.Schema` instance
         """
-        assert issubclass(Model, db.Model)
-        self.Model = Model
-        self.indexable = getattr(Model, "__indexable__", False)
-        self.index_args = getattr(Model, "__indexation_args__", {})
+        assert issubclass(model_class, db.Model)
+        self.model_class = model_class
+        self.indexable = getattr(model_class, "__indexable__", False)
+        self.index_to = self.get_index_to(model_class)
         self.doc_attrs = {}
         if self.indexable:
-            self._build_doc_attrs(Model, schema)
+            self._build_doc_attrs(model_class, schema)
 
-    def _build_doc_attrs(self, Model, schema):
-        mapper = sa.inspect(Model)
+    @staticmethod
+    def can_adapt(obj_cls: Any) -> bool:
+        return issubclass(obj_cls, db.Model)
+
+    def get_index_to(self, model_class: Type[Model]) -> Tuple:
+        result = []
+        classes = model_class.mro()
+        for cls in classes:
+            if hasattr(cls, "__index_to__"):
+                result += cls.__index_to__
+        return tuple(result)
+
+    def _build_doc_attrs(self, model_class: Type[Model], schema: Schema) -> None:
+        mapper = sa.inspect(model_class)
 
         args = self.doc_attrs
-        # any field not in schema will be stored here. After all field have been
-        # discovered we add missing ones
+        # Any field not in schema will be stored here.
+        # After all field have been discovered, we add the missing ones.
         field_definitions = {}
 
-        def setup_field(attr_name, field_name):
+        def setup_field(
+            attr_name: str, field_name: Union[Tuple[str, Union[type, ID]], str]
+        ) -> None:
             field_def = False
-            if not isinstance(field_name, string_types):
+            if not isinstance(field_name, str):
                 field_name, field_def = field_name
 
             if field_name not in schema:
@@ -117,11 +126,11 @@ class SAAdapter(SchemaAdapter):
 
             # attrgetter offers dotted name support. Useful for attributes on
             # related objects.
-            args.setdefault(field_name, {})[name] = attrgetter(name)
+            args.setdefault(field_name, {})[attr_name] = attrgetter(attr_name)
 
         # model level definitions
-        for name, field_names in self.index_args.get("index_to", ()):
-            if isinstance(field_names, string_types):
+        for name, field_names in self.index_to:
+            if isinstance(field_names, str):
                 field_names = (field_names,)
             for field_name in field_names:
                 setup_field(name, field_name)
@@ -135,7 +144,7 @@ class SAAdapter(SchemaAdapter):
                 continue
 
             index_to = info.get("index_to", (name,))
-            if isinstance(index_to, string_types):
+            if isinstance(index_to, str):
                 index_to = (index_to,)
 
             for field_name in index_to:
@@ -151,29 +160,31 @@ class SAAdapter(SchemaAdapter):
 
             logger.debug(
                 "Adding field to schema:\n" "  Model: %s\n" '  Field: "%s" %s',
-                Model._object_type(),
+                model_class._object_type(),
                 field_name,
                 field_def,
             )
             schema.add(field_name, field_def)
 
-    def retrieve(self, pk, _session=None, **data):
+    def retrieve(
+        self, pk: int, _session: Optional[Session] = None, **data: Any
+    ) -> Entity:
         if _session is None:
             _session = db.session()
-        return _session.query(self.Model).get(pk)
+        return _session.query(self.model_class).get(pk)
 
-    def get_document(self, obj):
-        kwargs = {}
+    def get_document(self, obj: Model) -> Dict[str, Any]:
+        result: Dict[str, Any] = {}
         if not self.indexable:
-            return kwargs
+            return result
 
-        # cache because the same attribute may be needed by many fields, i.e
-        # "title" on "title" field and "full_text" field for example
+        # Cache because the same attribute may be needed by many fields, i.e
+        # "title" on "title" field and "full_text" field for example.
         cached = {}
-        # negative cache. Might be used especially with dotted names
+        # Negative cache. Might be used especially with dotted names.
         missed = set()
 
-        for field, attrs in self.doc_attrs.items():
+        for field_name, attrs in self.doc_attrs.items():
             values = []
 
             for a, getter in attrs.items():
@@ -199,8 +210,8 @@ class SAAdapter(SchemaAdapter):
 
             if values:
                 if len(values) == 1:
-                    kwargs[field] = values[0]
+                    result[field_name] = values[0]
                 else:
-                    kwargs[field] = " ".join(values)
+                    result[field_name] = " ".join(values)
 
-        return kwargs
+        return result
