@@ -1,34 +1,31 @@
-# coding=utf-8
 """Indexing service for Abilian.
 
 Adds Whoosh indexing capabilities to SQLAlchemy models.
 
 Based on Flask-whooshalchemy by Karl Gyllstrom.
 
-:copyright: (c) 2013-2014 by Abilian SAS
+:copyright: (c) 2013-2019 by Abilian SAS
 :copyright: (c) 2012 by Stefane Fermigier
 :copyright: (c) 2012 by Karl Gyllstrom
 :license: BSD (see LICENSE.txt)
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 from inspect import isclass
 from pathlib import Path
-from typing import Set
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Tuple, \
+    Type, Union
 
 import sqlalchemy as sa
-import whoosh.query as wq
 from celery import shared_task
-from flask import _app_ctx_stack, appcontext_pushed, current_app, g
+from flask import Flask, _app_ctx_stack, appcontext_pushed, current_app, g
 from flask.globals import _lookup_app_object
 from flask_login import current_user
-from six import text_type
 from sqlalchemy import event
-from sqlalchemy.orm.session import Session
+from sqlalchemy.orm import Session
+from sqlalchemy.orm.unitofwork import UOWTransaction
+from whoosh import query as wq
 from whoosh.filedb.filestore import FileStorage, RamStorage
-from whoosh.index import FileIndex
+from whoosh.index import FileIndex, Index
 from whoosh.qparser import DisMaxParser
 from whoosh.writing import CLEAR, AsyncWriter
 
@@ -36,6 +33,7 @@ from abilian.core import signals
 from abilian.core.celery import safe_session
 from abilian.core.entities import Entity, Indexable
 from abilian.core.extensions import db
+from abilian.core.models import Model
 from abilian.core.models.subjects import Group, User
 from abilian.core.util import fqcn as base_fqcn
 from abilian.core.util import friendly_fqcn
@@ -45,6 +43,9 @@ from abilian.services.security import Anonymous, Authenticated, Role, security
 from .adapter import SAAdapter
 from .schema import DefaultSearchSchema, indexable_role
 
+if TYPE_CHECKING:
+    from abilian.app import Application
+
 logger = logging.getLogger(__name__)
 
 _pending_indexation_attr = "abilian_pending_indexation"
@@ -63,29 +64,31 @@ def url_for_hit(hit, default="#"):
         return default
 
 
-def fqcn(cls):
+def fqcn(cls: Any) -> str:
     if issubclass(cls, Entity):
         return cls.entity_type
     return base_fqcn(cls)
 
 
 class IndexServiceState(ServiceState):
-    def __init__(self, *args, **kwargs):
-        ServiceState.__init__(self, *args, **kwargs)
+    def __init__(
+        self, service: "WhooshIndexService", *args: Any, **kwargs: Any
+    ) -> None:
+        super().__init__(service, *args, **kwargs)
         self.whoosh_base = None
-        self.indexes = {}
-        self.indexed_classes = set()  # type: Set[type]
-        self.indexed_fqcn = set()
+        self.indexes: Dict[str, Index] = {}
+        self.indexed_classes: Set[type] = set()
+        self.indexed_fqcn: Set[str] = set()
         self.search_filter_funcs = []
         self.value_provider_funcs = []
         self.url_for_hit = url_for_hit
 
     @property
-    def to_update(self):
+    def to_update(self) -> List[Tuple[str, Entity]]:
         return _lookup_app_object(_pending_indexation_attr)
 
     @to_update.setter
-    def to_update(self, value):
+    def to_update(self, value: List) -> None:
         top = _app_ctx_stack.top
         if top is None:
             raise RuntimeError("working outside of application context")
@@ -99,26 +102,26 @@ class WhooshIndexService(Service):
     name = "indexing"
     AppStateClass = IndexServiceState
 
-    def __init__(self, *args, **kwargs):
-        Service.__init__(self, *args, **kwargs)
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
         self.adapters_cls = [SAAdapter]
         self.adapted = {}
         self.schemas = {"default": DefaultSearchSchema()}
         self._listening = False
 
-    def init_app(self, app):
-        Service.init_app(self, app)
+    def init_app(self, app: "Application") -> None:
+        super().init_app(app)
         state = app.extensions[self.name]
 
         whoosh_base = Path(app.config.get("WHOOSH_BASE", "whoosh"))
 
         if not whoosh_base.is_absolute():
-            whoosh_base = app.instance_path / whoosh_base
+            whoosh_base = Path(app.instance_path) / whoosh_base
 
         if not whoosh_base.is_dir():
             whoosh_base.mkdir(parents=True)
 
-        state.whoosh_base = text_type(whoosh_base.resolve())
+        state.whoosh_base = str(whoosh_base.resolve())
 
         if not self._listening:
             event.listen(Session, "after_flush", self.after_flush)
@@ -128,7 +131,7 @@ class WhooshIndexService(Service):
         appcontext_pushed.connect(self.clear_update_queue, app)
         signals.register_js_api.connect(self._do_register_js_api)
 
-    def _do_register_js_api(self, sender):
+    def _do_register_js_api(self, sender: "Application") -> None:
         app = sender
         js_api = app.js_api.setdefault("search", {})
         js_api["object_types"] = self.searchable_object_types()
@@ -153,16 +156,16 @@ class WhooshIndexService(Service):
         """
         self.app_state.value_provider_funcs.append(func)
 
-    def clear_update_queue(self, app=None):
+    def clear_update_queue(self, app: Optional[Flask] = None) -> None:
         self.app_state.to_update = []
 
-    def start(self, **kw):
-        Service.start(self)
+    def start(self, ignore_state: bool = False) -> None:
+        super().start(ignore_state)
         self.register_classes()
         self.init_indexes()
         self.clear_update_queue()
 
-    def init_indexes(self):
+    def init_indexes(self) -> None:
         """Create indexes for schemas."""
         state = self.app_state
 
@@ -173,7 +176,7 @@ class WhooshIndexService(Service):
                 index_path = (Path(state.whoosh_base) / name).absolute()
                 if not index_path.exists():
                     index_path.mkdir(parents=True)
-                storage = FileStorage(text_type(index_path))
+                storage = FileStorage(str(index_path))
 
             if storage.index_exists(name):
                 index = FileIndex(storage, schema, name)
@@ -182,7 +185,7 @@ class WhooshIndexService(Service):
 
             state.indexes[name] = index
 
-    def clear(self):
+    def clear(self) -> None:
         """Remove all content from indexes, and unregister all classes.
 
         After clear() the service is stopped. It must be started again
@@ -203,11 +206,11 @@ class WhooshIndexService(Service):
         if self.running:
             self.stop()
 
-    def index(self, name="default"):
+    def index(self, name: str = "default") -> Index:
         return self.app_state.indexes[name]
 
     @property
-    def default_search_fields(self):
+    def default_search_fields(self) -> Dict[str, float]:
         """Return default field names and boosts to be used for searching.
 
         Can be configured with `SEARCH_DEFAULT_BOOSTS`
@@ -215,9 +218,9 @@ class WhooshIndexService(Service):
         config = current_app.config.get("SEARCH_DEFAULT_BOOSTS")
         if not config:
             config = {"name": 1.5, "name_prefix": 1.3, "description": 1.3, "text": 1.0}
-            return config
+        return config
 
-    def searchable_object_types(self):
+    def searchable_object_types(self) -> List:
         """List of (object_types, friendly name) present in the index."""
         try:
             idx = self.index()
@@ -234,14 +237,14 @@ class WhooshIndexService(Service):
 
     def search(
         self,
-        q,
-        index="default",
-        fields=None,
-        Models=(),
-        object_types=(),
-        prefix=True,
-        facet_by_type=None,
-        **search_args
+        q: str,
+        index: str = "default",
+        fields: Optional[Dict[str, float]] = None,
+        Models: Tuple[Type[Model]] = (),
+        object_types: Tuple[str] = (),
+        prefix: bool = True,
+        facet_by_type: None = None,
+        **search_args,
     ):
         """Interface to search indexes.
 
@@ -354,7 +357,7 @@ class WhooshIndexService(Service):
     def search_for_class(self, query, cls, index="default", **search_args):
         return self.search(query, Models=(fqcn(cls),), index=index, **search_args)
 
-    def register_classes(self):
+    def register_classes(self) -> None:
         state = self.app_state
         classes = (
             cls
@@ -365,7 +368,7 @@ class WhooshIndexService(Service):
             if cls not in state.indexed_classes:
                 self.register_class(cls, app_state=state)
 
-    def register_class(self, cls, app_state=None):
+    def register_class(self, cls: type, app_state: IndexServiceState = None) -> None:
         """Register a model class."""
         state = app_state if app_state is not None else self.app_state
 
@@ -380,7 +383,7 @@ class WhooshIndexService(Service):
         state.indexed_classes.add(cls)
         state.indexed_fqcn.add(cls_fqcn)
 
-    def after_flush(self, session, flush_context):
+    def after_flush(self, session: Session, flush_context: UOWTransaction) -> None:
         if not self.running or session is not db.session():
             return
 
@@ -400,7 +403,7 @@ class WhooshIndexService(Service):
 
                 to_update.append((key, obj))
 
-    def after_commit(self, session):
+    def after_commit(self, session: Session) -> None:
         """Any db updates go through here.
 
         We check if any of these models have ``__searchable__`` fields,
@@ -437,13 +440,13 @@ class WhooshIndexService(Service):
             index_update.apply_async(kwargs={"index": "default", "items": items})
         self.clear_update_queue()
 
-    def get_document(self, obj, adapter=None):
+    def get_document(self, obj: Entity, adapter: SAAdapter = None) -> Dict[str, Any]:
         if adapter is None:
             class_name = fqcn(obj.__class__)
             adapter = self.adapted.get(class_name)
 
         if adapter is None or not adapter.indexable:
-            return None
+            return {}
 
         document = adapter.get_document(obj)
 
@@ -477,7 +480,7 @@ class WhooshIndexService(Service):
         with index.writer() as writer:
             for obj in objects:
                 document = self.get_document(obj)
-                if document is None:
+                if not document:
                     continue
 
                 object_key = document["object_key"]
@@ -500,7 +503,7 @@ service = WhooshIndexService()
 
 
 @shared_task
-def index_update(index, items):
+def index_update(index: str, items: List[List[Union[Dict, int, str]]]) -> None:
     """
     :param:index: index name
     :param:items: list of (operation, full class name, primary key, data) tuples.
@@ -519,7 +522,7 @@ def index_update(index, items):
 
             # always delete. Whoosh manual says that 'update' is actually delete + add
             # operation
-            object_key = "{}:{}".format(cls_name, pk)
+            object_key = f"{cls_name}:{pk}"
             writer.delete_by_term("object_key", object_key)
 
             adapter = adapted.get(cls_name)
@@ -551,7 +554,7 @@ def index_update(index, items):
                     logger.error("writer.add_document(%r)", document, exc_info=True)
                     raise
                 updated.add(object_key)
-    except BaseException:
+    except Exception:
         writer.cancel()
         raise
 
@@ -576,5 +579,5 @@ class TestingStorage(RamStorage):
     tests are ran in parallel, including different abilian-based packages.
     """
 
-    def temp_storage(self, name=None):
+    def temp_storage(self, name: str = None) -> "TestingStorage":
         return TestingStorage()
