@@ -1,9 +1,7 @@
-# coding=utf-8
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any
 
 
-class PreferencePanel(object):
+class PreferencePanel:
     """Base class for preference panels.
 
     Currently, this class does nothing. I may be useful in the future
@@ -12,5 +10,14 @@ class PreferencePanel(object):
     will be removed.
     """
 
+    id: str
+    label: Any
+
     def is_accessible(self):
         return True
+
+    def get(self):
+        raise NotImplementedError
+
+    def post(self):
+        raise NotImplementedError
