@@ -1,4 +1,3 @@
-# coding=utf-8
 """User preference service.
 
 Notes:
@@ -7,23 +6,27 @@ Notes:
 - For application settings use
   :class:`abilian.services.settings.SettingsService`.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
-from flask import Blueprint, g, redirect, request, url_for
+from flask import Blueprint, Flask, g, redirect, request, url_for
 from flask_login import current_user
 from werkzeug.exceptions import InternalServerError
 
 from abilian.core import signals
 from abilian.core.extensions import db
+from abilian.core.models.subjects import User
 from abilian.i18n import _, _l
 from abilian.services.auth.service import user_menu
 from abilian.services.base import Service, ServiceState
+from abilian.services.preferences.panel import PreferencePanel
 from abilian.web.action import Endpoint
 from abilian.web.nav import BreadcrumbItem, NavItem
 
 from .models import UserPreference
 
+if TYPE_CHECKING:
+    from abilian.app import Application
+
 _PREF_NAV_ITEM = NavItem(
     "user",
     "preferences",
@@ -37,15 +40,16 @@ user_menu.insert(0, _PREF_NAV_ITEM)
 
 
 class PreferenceState(ServiceState):
-    panels = None
-    blueprint = None
-    blueprint_registered = False
+    panels: List["PreferencePanel"]
+    blueprint: Blueprint
+    blueprint_registered: bool
 
-    def __init__(self, *args, **kwargs):
-        ServiceState.__init__(self, *args, **kwargs)
+    def __init__(self, service: "PreferenceService", *args: Any, **kwargs: Any) -> None:
+        super().__init__(service, *args, **kwargs)
         self.panels = []
         self.nav_paths = {}
         self.breadcrumb_items = {}
+        self.blueprint_registered = False
 
 
 class PreferenceService(Service):
@@ -55,15 +59,15 @@ class PreferenceService(Service):
     name = "preferences"
     AppStateClass = PreferenceState
 
-    def init_app(self, app, *panels):
-        Service.init_app(self, app)
+    def init_app(self, app: "Application", *panels: Any) -> None:
+        super().init_app(app)
 
         with app.app_context():
             self.setup_blueprint(app)
             for panel in panels:
                 self.register_panel(panel)
 
-    def get_preferences(self, user=None):
+    def get_preferences(self, user: Optional[User] = None) -> Dict[str, Any]:
         """Return a string->value dictionnary representing the given user
         preferences.
 
@@ -73,7 +77,7 @@ class PreferenceService(Service):
             user = current_user
         return {pref.key: pref.value for pref in user.preferences}
 
-    def set_preferences(self, user=None, **kwargs):
+    def set_preferences(self, user: User = None, **kwargs: Any) -> None:
         """Set preferences from keyword arguments."""
         if user is None:
             user = current_user
@@ -86,7 +90,7 @@ class PreferenceService(Service):
                 d[k] = UserPreference(user=user, key=k, value=v)
                 db.session.add(d[k])
 
-    def clear_preferences(self, user=None):
+    def clear_preferences(self, user: User = None) -> None:
         """Clear the user preferences."""
         if user is None:
             user = current_user
@@ -96,7 +100,9 @@ class PreferenceService(Service):
         #  http://docs.sqlalchemy.org/en/rel_0_7/orm/session.html#deleting-from-collections
         user.preferences = []
 
-    def register_panel(self, panel, app=None):
+    def register_panel(
+        self, panel: PreferencePanel, app: Optional[Flask] = None
+    ) -> None:
         state = self.app_state if app is None else app.extensions[self.name]
         if state.blueprint_registered:
             raise ValueError(
@@ -107,7 +113,7 @@ class PreferenceService(Service):
         panel.preferences = self
         rule = "/" + getattr(panel, "path", panel.id)
         endpoint = panel.id
-        abs_endpoint = "preferences.{}".format(endpoint)
+        abs_endpoint = f"preferences.{endpoint}"
 
         if hasattr(panel, "get"):
             state.blueprint.add_url_rule(rule, endpoint, panel.get)
@@ -119,7 +125,7 @@ class PreferenceService(Service):
             label=panel.label, icon=None, url=Endpoint(abs_endpoint)
         )
 
-    def setup_blueprint(self, app):
+    def setup_blueprint(self, app: Flask) -> None:
         bp = self.app_state.blueprint = Blueprint(
             "preferences",
             __name__,
@@ -130,7 +136,7 @@ class PreferenceService(Service):
         # we need to delay blueprint registration to allow adding more panels during
         # initialization
         @signals.components_registered.connect_via(app)
-        def register_bp(app):
+        def register_bp(app: Flask) -> None:
             app.register_blueprint(bp)
             app.extensions[self.name].blueprint_registered = True
 
@@ -141,7 +147,7 @@ class PreferenceService(Service):
         bp.url_value_preprocessor(self.build_breadcrumbs)
 
         @bp.context_processor
-        def inject_menu():
+        def inject_menu() -> Dict[str, List[Dict[str, Any]]]:
             menu = []
             for panel in self.app_state.panels:
                 if not panel.is_accessible():
@@ -168,9 +174,9 @@ class PreferenceService(Service):
             for panel in self.app_state.panels:
                 if panel.is_accessible():
                     return redirect(url_for("preferences." + panel.id))
-            else:
-                # Should not happen.
-                raise InternalServerError()
+
+            # Should not happen.
+            raise InternalServerError()
 
     def build_breadcrumbs(self, endpoint, view_args):
         state = self.app_state
