@@ -1,14 +1,11 @@
-# coding=utf-8
 """Social graph API.
 
 May or may not be needed, since the followers / followees notions are
 already implemented in the core model.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
 
 
-class FollowService(object):
+class FollowService:
     def get_followers(self, user):
         pass
 
