@@ -1,7 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, print_function, unicode_literals
-
 from flask import current_app
 from flask_debugtoolbar.panels import DebugPanel
 
@@ -39,8 +36,8 @@ class SecurityInfoDebugPanel(DebugPanel):
             return _("No current object")
 
         try:
-            return "{}(id={})".format(obj.__class__.__name__, obj.id)
-        except BaseException:
+            return f"{obj.__class__.__name__}(id={obj.id})"
+        except Exception:
             return ""
 
     def title(self):
@@ -72,15 +69,13 @@ class SecurityInfoDebugPanel(DebugPanel):
         for r in roles:
             info = roles[r]
             info["groups"] = [
-                "{g} (id={g.id})".format(g=g)
+                f"{g} (id={{g.id}})"
                 for g in sorted(info["groups"], key=lambda g: g.name)
             ]
             users = sorted(
                 info["users"], key=lambda u: (u.last_name.lower(), u.first_name.lower())
             )
-            info["users"] = [
-                '{u} (id={u.id}, email="{u.email}")'.format(u=u) for u in users
-            ]
+            info["users"] = [f'{u} (id={{u.id}}, email="{{u.email}}")' for u in users]
 
         jinja_env = current_app.jinja_env
         jinja_env.filters.update(self.jinja_env.filters)
