@@ -1,29 +1,34 @@
-# coding=utf-8
 """Security service, manages roles and permissions."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from functools import wraps
 from itertools import chain
+from typing import TYPE_CHECKING, Any, Callable, Collection, Dict, FrozenSet, \
+    List, Optional, Set, Type, Union
 
 import sqlalchemy as sa
 from flask import g
 from flask_login import current_user
-from six import string_types, text_type
 from sqlalchemy import sql
-from sqlalchemy.orm import object_session, subqueryload
+from sqlalchemy.orm import Session, object_session, subqueryload
+from sqlalchemy.sql.selectable import Exists
 
 from abilian.core.entities import Entity
 from abilian.core.extensions import db
-from abilian.core.models.subjects import Group, User
+from abilian.core.extensions.login import AnonymousUser
+from abilian.core.models import Model
+from abilian.core.models.subjects import Group, Principal, User
 from abilian.core.util import unwrap
 from abilian.services import Service, ServiceState
 from abilian.services.security.models import CREATE, DELETE, MANAGE, \
     PERMISSIONS_ATTR, READ, WRITE, Admin
 from abilian.services.security.models import Anonymous as AnonymousRole
 from abilian.services.security.models import Authenticated, Creator, \
-    InheritSecurity, Manager, Owner, Permission, PermissionAssignment, \
-    Reader, Role, RoleAssignment, SecurityAudit, Writer
+    FolderishModel, InheritSecurity, Manager, Owner, Permission, \
+    PermissionAssignment, Reader, Role, RoleAssignment, SecurityAudit, \
+    Writer
+
+if TYPE_CHECKING:
+    from abilian.app import Application
+
 
 #: list of legacy supported permissions when not using :class:`Permission`
 #: instance
@@ -38,7 +43,7 @@ __all__ = [
 ]
 
 #: default security matrix
-DEFAULT_PERMISSION_ROLE = {}
+DEFAULT_PERMISSION_ROLE: Dict[Permission, FrozenSet[Role]] = {}
 prm = DEFAULT_PERMISSION_ROLE
 prm[MANAGE] = frozenset({Admin, Manager})
 prm[WRITE] = frozenset({Admin, Manager, Writer})
@@ -58,7 +63,7 @@ class SecurityServiceState(ServiceState):
     needs_db_flush = False
 
 
-def require_flush(fun):
+def require_flush(fun: Callable) -> Callable:
     """Decorator for methods that need to query security.
 
     It ensures all security related operations are flushed to DB, but
@@ -66,7 +71,7 @@ def require_flush(fun):
     """
 
     @wraps(fun)
-    def ensure_flushed(service, *args, **kwargs):
+    def ensure_flushed(service: "SecurityService", *args: Any, **kwargs: Any) -> Any:
         if service.app_state.needs_db_flush:
             session = db.session()
             if not session._flushing and any(
@@ -82,7 +87,9 @@ def require_flush(fun):
     return ensure_flushed
 
 
-def query_pa_no_flush(session, permission, role, obj):
+def query_pa_no_flush(
+    session: Session, permission: Permission, role: Role, obj: Optional[Model]
+):
     """Query for a :class:`PermissionAssignment` using `session` without any
     `flush()`.
 
@@ -138,27 +145,27 @@ class SecurityService(Service):
     name = "security"
     AppStateClass = SecurityServiceState
 
-    def init_app(self, app):
-        Service.init_app(self, app)
+    def init_app(self, app: "Application") -> None:
+        super().init_app(app)
         state = app.extensions[self.name]
         state.use_cache = True
 
-    def _needs_flush(self):
+    def _needs_flush(self) -> None:
         """Mark next security queries needs DB flush to have up to date
         information."""
         self.app_state.needs_db_flush = True
 
-    def clear(self):
+    def clear(self) -> None:
         pass
 
-    def _current_user_manager(self, session=None):
+    def _current_user_manager(self, session: Session = None) -> User:
         """Return the current user, or SYSTEM user."""
         if session is None:
             session = db.session()
 
         try:
             user = g.user
-        except BaseException:
+        except Exception:
             return session.query(User).get(0)
 
         if sa.orm.object_session(user) is not session:
@@ -182,7 +189,7 @@ class SecurityService(Service):
         )
 
     # inheritance
-    def set_inherit_security(self, obj, inherit_security):
+    def set_inherit_security(self, obj: FolderishModel, inherit_security: bool) -> None:
         assert isinstance(obj, InheritSecurity)
         assert isinstance(obj, Entity)
 
@@ -211,7 +218,12 @@ class SecurityService(Service):
     # Roles-related API.
     #
     @require_flush
-    def get_roles(self, principal, object=None, no_group_roles=False):
+    def get_roles(
+        self,
+        principal: Union[AnonymousUser, Group, User],
+        object: Optional[Model] = None,
+        no_group_roles: bool = False,
+    ) -> List[Role]:
         """Get all the roles attached to given `principal`, on a given
         `object`.
 
@@ -252,8 +264,14 @@ class SecurityService(Service):
 
     @require_flush
     def get_principals(
-        self, role, anonymous=True, users=True, groups=True, object=None, as_list=True
-    ):
+        self,
+        role: Role,
+        anonymous: bool = True,
+        users: bool = True,
+        groups: bool = True,
+        object: Optional[Model] = None,
+        as_list: bool = True,
+    ) -> Collection[Principal]:
         """Return all users which are assigned given role."""
         if not isinstance(role, Role):
             role = Role(role)
@@ -282,7 +300,7 @@ class SecurityService(Service):
         return list(principals)
 
     @require_flush
-    def _all_roles(self, principal):
+    def _all_roles(self, principal: Principal) -> Dict[Optional[str], Set[Role]]:
         query = (
             db.session.query(RoleAssignment.object_id, RoleAssignment.role)
             .outerjoin(Entity)
@@ -305,39 +323,45 @@ class SecurityService(Service):
             if object_id is None:
                 object_key = None
             else:
-                object_key = "{}:{}".format(object_type, object_id)
+                object_key = f"{object_type}:{object_id}"
             all_roles.setdefault(object_key, set()).add(role)
 
         return all_roles
 
-    def _role_cache(self, principal):
+    def _role_cache(self, principal: Principal) -> Dict[Optional[str], Set[Role]]:
         if not self._has_role_cache(principal):
             # FIXME: should call _fill_role_cache?
             principal.__roles_cache__ = {}
 
         return principal.__roles_cache__
 
-    def _has_role_cache(self, principal):
+    def _has_role_cache(self, principal: Principal) -> bool:
         return hasattr(principal, "__roles_cache__")
 
-    def _set_role_cache(self, principal, cache):
+    def _set_role_cache(
+        self, principal: Principal, cache: Dict[Optional[str], Set[Role]]
+    ) -> None:
         principal.__roles_cache__ = cache
 
-    def _fill_role_cache(self, principal, overwrite=False):
+    def _fill_role_cache(
+        self, principal: Principal, overwrite: bool = False
+    ) -> Dict[Optional[str], Set[Role]]:
         """Fill role cache for `principal` (User or Group), in order to avoid
         too many queries when checking role access with 'has_role'.
 
         Return role_cache of `principal`
         """
         if not self.app_state.use_cache:
-            return None
+            return {}
 
         if not self._has_role_cache(principal) or overwrite:
             self._set_role_cache(principal, self._all_roles(principal))
         return self._role_cache(principal)
 
     @require_flush
-    def _fill_role_cache_batch(self, principals, overwrite=False):
+    def _fill_role_cache_batch(
+        self, principals: Collection[Principal], overwrite: bool = False
+    ) -> None:
         """Fill role cache for `principals` (Users and/or Groups), in order to
         avoid too many queries when checking role access with 'has_role'."""
         if not self.app_state.use_cache:
@@ -363,13 +387,13 @@ class SecurityService(Service):
 
         filter_cond = []
         if users:
-            filter_cond.append(RoleAssignment.user_id.in_((u.id for u in users)))
+            filter_cond.append(RoleAssignment.user_id.in_(u.id for u in users))
         if groups:
-            filter_cond.append(RoleAssignment.group_id.in_((g.id for g in groups)))
+            filter_cond.append(RoleAssignment.group_id.in_(g.id for g in groups))
 
         query = query.filter(sql.or_(*filter_cond))
-        ra_users = {}
-        ra_groups = {}
+        ra_users: Dict[User, Dict[str, Set[Role]]] = {}
+        ra_groups: Dict[Group, Dict[str, Set[Role]]] = {}
         for ra in query.all():
             if ra.user:
                 all_roles = ra_users.setdefault(ra.user, {})
@@ -377,7 +401,7 @@ class SecurityService(Service):
                 all_roles = ra_groups.setdefault(ra.group, {})
 
             object_key = (
-                "{}:{:d}".format(ra.object.entity_type, ra.object_id)
+                f"{ra.object.entity_type}:{ra.object_id:d}"
                 if ra.object is not None
                 else None
             )
@@ -395,7 +419,7 @@ class SecurityService(Service):
 
             self._set_role_cache(user, all_roles)
 
-    def _clear_role_cache(self, principal):
+    def _clear_role_cache(self, principal: Principal) -> None:
         if hasattr(principal, "__roles_cache__"):
             del principal.__roles_cache__
 
@@ -404,7 +428,12 @@ class SecurityService(Service):
                 if hasattr(u, "__roles_cache__"):
                     del u.__roles_cache__
 
-    def has_role(self, principal, role, object=None):
+    def has_role(
+        self,
+        principal: Principal,
+        role: Union[Collection[Role], Role, str],
+        object: Optional[Model] = None,
+    ) -> bool:
         """True if `principal` has `role` (either globally, if `object` is
         None, or on the specific `object`).
 
@@ -425,7 +454,7 @@ class SecurityService(Service):
         if not self.running:
             return True
 
-        if isinstance(role, (Role, string_types)):
+        if isinstance(role, (Role, (str,))):
             role = (role,)
 
         # admin & manager always have role
@@ -454,7 +483,7 @@ class SecurityService(Service):
 
         if object:
             assert isinstance(object, Entity)
-            object_key = "{}:{}".format(object.object_type, text_type(object.id))
+            object_key = f"{object.object_type}:{str(object.id)}"
             if Creator in role:
                 if object.creator == principal:
                     return True
@@ -475,7 +504,9 @@ class SecurityService(Service):
         roles |= all_roles.get(object_key, set())
         return len(valid_roles & roles) > 0
 
-    def grant_role(self, principal, role, obj=None):
+    def grant_role(
+        self, principal: Principal, role: Union[Role, str], obj: Optional[Model] = None
+    ) -> None:
         """Grant `role` to `user` (either globally, if `obj` is None, or on the
         specific `obj`)."""
         assert principal
@@ -535,9 +566,15 @@ class SecurityService(Service):
         if hasattr(principal, "__roles_cache__"):
             del principal.__roles_cache__
 
-    def ungrant_role(self, principal, role, object=None):
+    def ungrant_role(
+        self,
+        principal: Principal,
+        role: Union[Role, str],
+        object: Optional[Model] = None,
+    ) -> None:
         """Ungrant `role` to `user` (either globally, if `object` is None, or
         on the specific `object`)."""
+
         assert principal
         principal = unwrap(principal)
         session = object_session(object) if object is not None else db.session
@@ -580,7 +617,7 @@ class SecurityService(Service):
         self._clear_role_cache(principal)
 
     @require_flush
-    def get_role_assignements(self, obj):
+    def get_role_assignements(self, obj: Model) -> List:
         session = object_session(obj) if obj is not None else db.session
         if not session:
             session = db.session()
@@ -605,7 +642,14 @@ class SecurityService(Service):
     #
     # Permission API, currently hardcoded
     #
-    def has_permission(self, user, permission, obj=None, inherit=False, roles=None):
+    def has_permission(
+        self,
+        user: User,
+        permission: Union[Permission, str],
+        obj: Optional[Model] = None,
+        inherit: bool = False,
+        roles: Union[None, Role, str, List[Union[Role, str]]] = None,
+    ) -> bool:
         """
         :param obj: target object to check permissions.
         :param inherit: check with permission inheritance. By default, check only
@@ -649,7 +693,7 @@ class SecurityService(Service):
         # FIXME: obj.__class__ could define default permisssion matrix too
 
         if roles is not None:
-            if isinstance(roles, (Role,) + string_types):
+            if isinstance(roles, (Role, str)):
                 roles = (roles,)
 
             for r in roles:
@@ -674,14 +718,17 @@ class SecurityService(Service):
         self._fill_role_cache_batch(principals)
 
         return any(
-            (
-                self.has_role(principal, valid_roles, item)
-                for principal in principals
-                for item in checked_objs
-            )
+            self.has_role(principal, valid_roles, item)
+            for principal in principals
+            for item in checked_objs
         )
 
-    def query_entity_with_permission(self, permission, user=None, Model=Entity):
+    def query_entity_with_permission(
+        self,
+        permission: Permission,
+        user: Optional[User] = None,
+        Model: Type[Model] = Entity,
+    ) -> Exists:
         """Filter a query on an :class:`Entity` or on of its subclasses.
 
         Usage::
@@ -766,7 +813,9 @@ class SecurityService(Service):
 
         return filter_expr
 
-    def get_permissions_assignments(self, obj=None, permission=None):
+    def get_permissions_assignments(
+        self, obj: Optional[Model] = None, permission: Optional[Permission] = None
+    ) -> Dict[Permission, Set[Role]]:
         """
         :param permission: return only roles having this permission
 
@@ -790,13 +839,15 @@ class SecurityService(Service):
         if permission:
             pa = pa.filter(PermissionAssignment.permission == permission)
 
-        results = {}
+        results: Dict[permission, Set[Role]] = {}
         for permission, role in pa.yield_per(1000):
             results.setdefault(permission, set()).add(role)
 
         return results
 
-    def add_permission(self, permission, role, obj=None):
+    def add_permission(
+        self, permission: Permission, role: Role, obj: Optional[Model] = None
+    ) -> None:
         session = None
         if obj is not None:
             session = object_session(obj)
@@ -812,7 +863,9 @@ class SecurityService(Service):
         # do it in any case: it could have been found in session.deleted
         session.add(pa)
 
-    def delete_permission(self, permission, role, obj=None):
+    def delete_permission(
+        self, permission: Permission, role: Role, obj: Optional[Model] = None
+    ) -> None:
         session = None
         if obj is not None:
             session = object_session(obj)
@@ -828,7 +881,13 @@ class SecurityService(Service):
                 # this seems to be required with sqlalchemy > 0.9
                 session.expire(obj, [PERMISSIONS_ATTR])
 
-    def filter_with_permission(self, user, permission, obj_list, inherit=False):
+    def filter_with_permission(
+        self,
+        user: User,
+        permission: Union[Permission, str],
+        obj_list: List[Model],
+        inherit=False,
+    ):
         user = unwrap(user)
         return [
             obj
