@@ -1,36 +1,34 @@
-# coding=utf-8
-""""""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import json
 from datetime import timedelta
-from typing import Callable, Dict, Optional
+from typing import Any, Callable, Dict, Optional
 
 import sqlalchemy as sa
-from six import text_type
 
 from abilian.core.extensions import db
 
-__all__ = ["Setting"]
+__all__ = ["Setting", "empty_value"]
 
 
-class TransformerRegistry(object):
-    def __init__(self):
-        self.encoders = {}  # type: Dict[text_type, Optional[Callable]]
-        self.decoders = {}  # type: Dict[text_type, Optional[Callable]]
+class TransformerRegistry:
+    def __init__(self) -> None:
+        self.encoders: Dict[str, Optional[Callable]] = {}
+        self.decoders: Dict[str, Optional[Callable]] = {}
 
-    def encode(self, type_, value):
-        # bytes is str for python2
-        return self.encoders.get(type_, bytes)(value)
+    def encode(self, type_: str, value: Any) -> str:
+        return self.encoders.get(type_, str)(value)
 
-    def decode(self, type_, value):
+    def decode(self, type_: str, value: str) -> Any:
         decoder = self.decoders.get(type_)
         if decoder is not None:
             value = decoder(value)
         return value
 
-    def register(self, type_, encoder=None, decoder=None):
+    def register(
+        self,
+        type_: str,
+        encoder: Optional[Callable] = None,
+        decoder: Optional[Callable] = None,
+    ) -> None:
         assert type_
         assert any((encoder, decoder))
         if encoder:
@@ -42,19 +40,16 @@ class TransformerRegistry(object):
 _transformers = TransformerRegistry()
 
 
-class _EmptyValue(object):
+class _EmptyValue:
     def __bool__(self):
         return False
 
-    # Py2 compat
-    __nonzero__ = __bool__
-
     def __repr__(self):
         return "<Empty Value>"
 
 
-#: marker for emptyness, to distinguish from None
-EmptyValue = _EmptyValue()
+#: marker for emptiness, to distinguish from None
+empty_value = _EmptyValue()
 
 
 class Setting(db.Model):
@@ -71,74 +66,85 @@ class Setting(db.Model):
     #: if that's what you need. Type must be set before setting `value`
     _type = sa.Column("type", sa.String(length=1000), nullable=False)
 
+    _value = sa.Column("value", sa.Text())
+
     @property
-    def type(self):
+    def type(self) -> str:
         return self._type
 
     @type.setter
-    def type(self, type_):
+    def type(self, type_: str) -> None:
         if not (
             type_ in self.transformers.encoders and type_ in self.transformers.decoders
         ):
             raise ValueError(
-                'Invalid type "{}": no encoder and/or decoder registered'.format(type_)
+                f'Invalid type "{type_}": no encoder and/or decoder registered'
             )
         self._type = type_
 
-    _value = sa.Column("value", sa.Text())
-
     @property
     def value(self):
         if self._value is None:
-            return EmptyValue
+            return empty_value
+
+        assert isinstance(self._value, str)
         return self.transformers.decode(self.type, self._value)
 
     @value.setter
     def value(self, value):
         assert self.type
         self._value = self.transformers.encode(self.type, value)
+        assert isinstance(self._value, str)
 
 
 register = _transformers.register
 
 
-def from_int(i):
-    return "{}".format(i).encode()
+def from_int(i: int) -> str:
+    return f"{i}"
 
 
 register("int", from_int, int)
 
 
-def from_bool(b):
+def from_bool(b: bool) -> str:
     return "true" if b else "false"
 
 
-def to_bool(s):
+def to_bool(s: str) -> bool:
     return s == "true"
 
 
 register("bool", from_bool, to_bool)
 
 
-def from_unicode(s):
-    return text_type(s).encode("utf-8")
+def from_unicode(s: str) -> str:
+    return s
 
 
-def to_unicode(s):
-    if isinstance(s, text_type):
-        return s
-    return s.decode("utf-8")
+def to_unicode(s: str) -> str:
+    return s
 
 
 register("string", from_unicode, to_unicode)
-register("json", json.dumps, json.loads)  # FIXME: checks for dump/load?
 
 
-def from_timedelta(s):
+def from_obj(o: Any) -> str:
+    return json.dumps(o)
+
+
+def to_obj(s: str) -> Any:
+    return json.loads(s)
+
+
+register("json", from_obj, to_obj)
+
+
+def from_timedelta(s: timedelta) -> str:
     return json.dumps({"days": s.days, "seconds": s.seconds})
 
 
-def to_timedelta(s):
+def to_timedelta(s: str):
     return timedelta(**json.loads(s))
 
 
