@@ -1,7 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function
-
 from .models import Vocabulary
 from .service import get_vocabulary, vocabularies
 
