@@ -1,7 +1,5 @@
-# coding=utf-8
 """Admin panel for vocabularies."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Callable
 
 from flask import g, redirect, render_template, request
 
@@ -18,7 +16,7 @@ from .forms import EditForm
 _MARKER = object()
 
 
-class ViewBase(object):
+class ViewBase:
     title = _l("Vocabulary entry")
     base_template = "admin/_base.html"
     Form = EditForm
@@ -28,7 +26,7 @@ class ViewBase(object):
         if self.Model is None:
             self.Model = kwargs.get("Model")
             if hasattr(self.Model, "__form__"):
-                self.Form = getattr(self.Model, "__form__")
+                self.Form = self.Model.__form__
         return args, kwargs
 
     def index_url(self):
@@ -50,7 +48,7 @@ class Edit(ViewBase, views.ObjectEdit):
 
 class Create(views.ObjectCreate, Edit):
     def init_object(self, args, kwargs):
-        args, kwargs = super(Create, self).init_object(args, kwargs)
+        args, kwargs = super().init_object(args, kwargs)
         self.obj.active = True  # do this because default value seems ignored?
         return args, kwargs
 
@@ -179,7 +177,7 @@ class VocabularyPanel(AdminPanel):
         }
         return render_template("admin/vocabularies.html", **ctx)
 
-    def install_additional_rules(self, add_url_rule):
+    def install_additional_rules(self, add_url_rule: Callable) -> None:
         panel_endpoint = "." + self.id
         group_base = "/<string:group>/"
         add_url_rule(group_base, endpoint="group", view_func=self.group_view)
