@@ -1,15 +1,12 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any, Dict, Tuple, Type
 
-import six
 import sqlalchemy as sa
 import sqlalchemy.event
 import sqlalchemy.ext
 import sqlalchemy.ext.declarative
 import sqlalchemy.orm
-from flask_sqlalchemy import BaseQuery
+from flask_sqlalchemy import BaseQuery, Model
 from sqlalchemy import Column
 
 from abilian.core.extensions import db
@@ -19,7 +16,7 @@ _BaseMeta = db.Model.__class__
 
 
 class VocabularyQuery(BaseQuery):
-    def active(self):
+    def active(self) -> "VocabularyQuery":
         """Returns only valid vocabulary items."""
         return self.filter_by(active=True)
 
@@ -31,7 +28,7 @@ class VocabularyQuery(BaseQuery):
         except sa.orm.exc.NoResultFound:
             return None
 
-    def by_position(self, position):
+    def by_position(self, position: int) -> None:
         """Like `.get()`, but by position number."""
         # don't use .first(), so that MultipleResultsFound can be raised
         try:
@@ -46,7 +43,12 @@ class _VocabularyMeta(_BaseMeta):
     Enforces `__tablename__`.
     """
 
-    def __new__(cls, name, bases, d):
+    def __new__(
+        cls: Type["_VocabularyMeta"],
+        name: str,
+        bases: Tuple[Type[Model], ...],
+        d: Dict[str, Any],
+    ) -> Type["BaseVocabulary"]:
         meta = d.get("Meta")
         tblprefix = "vocabulary_"
 
@@ -61,9 +63,7 @@ class _VocabularyMeta(_BaseMeta):
         return _BaseMeta.__new__(cls, name, bases, d)
 
 
-@six.python_2_unicode_compatible
-@six.add_metaclass(_VocabularyMeta)
-class BaseVocabulary(db.Model):
+class BaseVocabulary(db.Model, metaclass=_VocabularyMeta):
     """Base abstract class for vocabularies."""
 
     __abstract__ = True
@@ -85,7 +85,7 @@ class BaseVocabulary(db.Model):
     def __mapper_args__(cls):
         return {"order_by": [cls.__table__.c.position.asc()]}
 
-    class Meta(object):
+    class Meta:
         label = None
         group = None
 
@@ -139,16 +139,12 @@ _generated_vocabularies = []
 
 
 def Vocabulary(name, label=None, group=None):
-    if isinstance(name, bytes):
-        name = name.decode()
+    assert isinstance(name, str)
 
     cls_name = "Vocabulary" + name.capitalize()
-    if six.PY2:
-        cls_name = bytes(cls_name)
-
     _name, _label, _group = name.lower(), label, group
 
-    class Meta(object):
+    class Meta:
         name = _name
         label = _label
         group = _group
