@@ -1,8 +1,5 @@
-# coding=utf-8
 """Vocabularies service."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
+import typing
 from inspect import isclass
 
 import jinja2
@@ -11,6 +8,9 @@ from abilian.services.base import Service
 
 from .models import BaseVocabulary
 
+if typing.TYPE_CHECKING:
+    from abilian.app import Application
+
 
 def _vocabularies():
     return {
@@ -27,7 +27,7 @@ def _grouped_vocabularies():
     return by_group
 
 
-def get_vocabulary(name, group=""):
+def get_vocabulary(name: str, group: str = "") -> None:
     name = name.lower()
     vocs = _grouped_vocabularies()
     for voc in vocs.get(group, ()):
@@ -40,8 +40,8 @@ def get_vocabulary(name, group=""):
 class VocabularyService(Service):
     name = "vocabularies"
 
-    def init_app(self, app):
-        Service.init_app(self, app)
+    def init_app(self, app: "Application") -> None:
+        super().init_app(app)
         app.register_jinja_loaders(jinja2.PackageLoader(__name__))
 
     @property
@@ -52,7 +52,7 @@ class VocabularyService(Service):
     def grouped_vocabularies(self):
         return _grouped_vocabularies()
 
-    def get_vocabulary(self, name, group=""):
+    def get_vocabulary(self, name: str, group: str = "") -> None:
         return get_vocabulary(name, group=group)
 
 
