@@ -1,8 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import sqlalchemy as sa
 import sqlalchemy.exc
 from pytest import mark, raises
@@ -28,7 +24,7 @@ def test_vocabulary_factory(session):
     assert Voc.__tablename__ == "vocabulary_voc"
     assert Voc.Meta.name == "voc"
     assert Voc.Meta.label == "Voc"
-    assert Voc.Meta.group is ""
+    assert Voc.Meta.group == ""
     assert issubclass(Voc, BaseVocabulary)
 
 
@@ -79,21 +75,21 @@ def test_items(db, session):
     assert [i.label for i in query.values(PriorityVoc.label)] == default_ordering
 
     # test db-side constraint for non-empty labels
-    with raises(sa.exc.IntegrityError, message="Could insert an item with empty label"):
+    with raises(sa.exc.IntegrityError):
         with session.begin_nested():
             v = PriorityVoc(label="   ", position=6)
             session.add(v)
             session.flush()
 
     # test unique labels constraint
-    with raises(sa.exc.IntegrityError, message="Could insert duplicate label"):
+    with raises(sa.exc.IntegrityError):
         with session.begin_nested():
             v = PriorityVoc(label="Immediate")
             session.add(v)
             session.flush()
 
     # test unique position constraint
-    with raises(sa.exc.IntegrityError, message="Could insert duplicate position"):
+    with raises(sa.exc.IntegrityError):
         with session.begin_nested():
             v = PriorityVoc(label="New one", position=1)
             session.add(v)
