@@ -1,4 +1,3 @@
-# coding=utf-8
 """Configuration and injectable fixtures for Pytest.
 
 Can be reused (and overriden) by adding::
@@ -7,47 +6,45 @@ Can be reused (and overriden) by adding::
 
 to your `conftest.py`.
 
-Supposed to replace the too-complex current UnitTest-based testing
-framework. -> DI and functions over complex inheritance hierarchies FTW!
+Replaced the too-complex UnitTest-based testing framework.
+-> DI and functions over complex inheritance hierarchies FTW!
 """
-
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
-from typing import Any
+from typing import Any, Iterator
 
 from flask import Flask
 from flask.ctx import AppContext, RequestContext
 from flask.testing import FlaskClient
-from flask_sqlalchemy import SQLAlchemy
 from pytest import fixture
+from sqlalchemy.orm import Session
 
 from abilian.app import create_app
 from abilian.core.models.subjects import User
+from abilian.core.sqlalchemy import SQLAlchemy
 from abilian.testing.util import cleanup_db, ensure_services_started, \
     stop_all_services
 
 
 class TestConfig:
     TESTING = True
+    DEBUG = True
+    SECRET_KEY = "SECRET"
     SERVER_NAME = "localhost.localdomain"
     CELERY_ALWAYS_EAGER = True  # run tasks locally, no async
     CELERY_EAGER_PROPAGATES_EXCEPTIONS = True
     MAIL_SENDER = "tester@example.com"
     SITE_NAME = "Abilian Test"
-    CSRF_ENABLED = True
-    WTF_CSRF_ENABLED = True
+    # WTF_CSRF_ENABLED = True
+    WTF_CSRF_ENABLED = False
     BABEL_ACCEPT_LANGUAGES = ["en", "fr"]
 
 
 @fixture
-def config():
+def config() -> type:
     return TestConfig
 
 
 @fixture
-def app(config):
-    # type: (Any) -> Flask
+def app(config: Any) -> Flask:
     # We currently return a fresh app for each test.
     # Using session-scoped app doesn't currently work.
     # Note: the impact on speed is minimal.
@@ -55,28 +52,25 @@ def app(config):
 
 
 @fixture
-def app_context(app):
-    # type: (Flask) -> AppContext
+def app_context(app: Flask) -> Iterator[AppContext]:
     with app.app_context() as ctx:
         yield ctx
 
 
 @fixture
-def test_request_context(app):
-    # type: (Flask) -> RequestContext
+def test_request_context(app: Flask) -> Iterator[RequestContext]:
     with app.test_request_context() as ctx:
         yield ctx
 
 
 @fixture
-def req_ctx(app):
-    # type: (Flask) -> RequestContext
-    with app.test_request_context() as req_ctx:
-        yield req_ctx
+def req_ctx(app: Flask) -> Iterator[RequestContext]:
+    with app.test_request_context() as _req_ctx:
+        yield _req_ctx
 
 
 @fixture
-def db(app_context):
+def db(app_context: AppContext) -> Iterator[SQLAlchemy]:
     """Return a fresh db for each test."""
     from abilian.core.extensions import db
 
@@ -93,25 +87,23 @@ def db(app_context):
 
 
 @fixture
-def session(db):
+def session(db: SQLAlchemy) -> Session:
     return db.session
 
 
 @fixture
-def db_session(db):
+def db_session(db: SQLAlchemy) -> Session:
     return db.session
 
 
 @fixture
-def client(app):
-    # type: (Flask) -> FlaskClient
+def client(app: Flask) -> FlaskClient:
     """Return a Web client, used for testing."""
     return app.test_client()
 
 
 @fixture
-def user(db):
-    # type: (SQLAlchemy) -> User
+def user(db: SQLAlchemy) -> User:
     user = User(
         first_name="Joe",
         last_name="Test",
@@ -125,8 +117,7 @@ def user(db):
 
 
 @fixture
-def admin_user(db):
-    # type: (SQLAlchemy) -> User
+def admin_user(db: SQLAlchemy) -> User:
     user = User(
         first_name="Jim",
         last_name="Admin",
@@ -141,8 +132,7 @@ def admin_user(db):
 
 
 @fixture
-def login_user(user, client):
-    # type: (User, FlaskClient) -> User
+def login_user(user: User, client: FlaskClient) -> User:
     with client.session_transaction() as session:
         session["user_id"] = user.id
 
@@ -150,8 +140,7 @@ def login_user(user, client):
 
 
 @fixture
-def login_admin(admin_user, client):
-    # type: (User, FlaskClient) -> User
+def login_admin(admin_user: User, client: FlaskClient) -> User:
     with client.session_transaction() as session:
         session["user_id"] = admin_user.id
 
