@@ -1,6 +1,3 @@
-# coding=utf-8
-from __future__ import absolute_import, print_function, unicode_literals
-
 import json
 import os
 import subprocess
@@ -48,9 +45,7 @@ def assert_valid(response):
         assert_json_valid(response)
 
     else:
-        raise AssertionError("Unknown mime type: " + response.mimetype)
-
-    return
+        raise AssertionError(f"Unknown mime type: {response.mimetype}")
 
 
 def assert_html_valid(response):
@@ -69,7 +64,7 @@ def assert_html_valid_using_htmlhint(response):
         except subprocess.CalledProcessError as e:
             print("htmllhint output:")
             print(e.output)
-            msg = "HTML was not valid for URL: {}".format(request.url)
+            msg = f"HTML was not valid for URL: {request.url}"
             raise ValidationError(msg)
 
 
@@ -94,7 +89,7 @@ def assert_html_valid_using_external_service(response):
             detail = "on line {} [{}]\n{}".format(
                 message["lastLine"], message["extract"], message["message"]
             )
-            msg = "Got a validation error for {}:\n{}".format(request.url, detail)
+            msg = f"Got a validation error for {request.url}:\n{detail}"
             raise ValidationError(msg)
 
 
@@ -102,6 +97,6 @@ def assert_json_valid(response):
     # type: (Response) -> None
     try:
         json.loads(response.data)
-    except BaseException:
-        msg = "JSON was not valid for URL: {}".format(request.url)
+    except Exception:
+        msg = f"JSON was not valid for URL: {request.url}"
         raise ValidationError(msg)
