@@ -1,12 +1,8 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from datetime import datetime, timedelta
+from typing import Any, Dict, List, Tuple
 
 import pandas as pd
-import six
 import sqlalchemy as sa
 from flask import render_template
 from numpy import sum as numpysum
@@ -16,8 +12,7 @@ from abilian.core.models.subjects import User
 from abilian.i18n import _, _l
 from abilian.services.audit import CREATION, AuditEntry
 from abilian.services.auth.models import LoginSession
-
-from ..panel import AdminPanel
+from abilian.web.admin import AdminPanel
 
 
 class DashboardPanel(AdminPanel):
@@ -26,7 +21,7 @@ class DashboardPanel(AdminPanel):
     label = _l("Dashboard")
     icon = "eye-open"
 
-    def get(self):
+    def get(self) -> str:
         login_entries = LoginSession.query.order_by(LoginSession.started_at.asc()).all()
         # .options(sa.orm.joinedload(LoginSession.user))
         daily, weekly, monthly = uniquelogins(login_entries)
@@ -67,7 +62,7 @@ class DashboardPanel(AdminPanel):
         )
 
 
-def stats_since(dt):
+def stats_since(dt: timedelta) -> Dict[str, int]:
     new_members = new_documents = new_messages = 0
     after_date = datetime.utcnow() - dt
     session = db.session()
@@ -112,7 +107,7 @@ def unix_time_millis(dt):
     return (dt - epoch).total_seconds() * 1000.0
 
 
-def newlogins(sessions):
+def newlogins(sessions: List[Any]) -> Tuple[List[Any], List[Any]]:
     """Brand new logins each day, and total of users each day.
 
     :return: data, total
@@ -149,7 +144,7 @@ def newlogins(sessions):
     return data, total
 
 
-def uniquelogins(sessions):
+def uniquelogins(sessions: List[Any]) -> Tuple[List[Any], List[Any], List[Any]]:
     """Unique logins per days/weeks/months.
 
     :return: daily, weekly, monthly
@@ -192,7 +187,7 @@ def uniquelogins(sessions):
     weekly_serie = daily_serie.groupby(pd.Grouper(freq="W")).aggregate(numpysum)
     monthly_serie = daily_serie.groupby(pd.Grouper(freq="M")).aggregate(numpysum)
 
-    for date, value in six.iteritems(weekly_serie):
+    for date, value in weekly_serie.items():
         try:
             value = int(value)
         except ValueError:
@@ -200,7 +195,7 @@ def uniquelogins(sessions):
         date_epoch = unix_time_millis(date)
         weekly.append({"x": date_epoch, "y": value})
 
-    for date, value in six.iteritems(monthly_serie):
+    for date, value in monthly_serie.items():
         try:
             value = int(value)
         except ValueError:
