@@ -1,8 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import sqlalchemy as sa
 from wtforms.fields import BooleanField, StringField
 from wtforms.validators import ValidationError
