@@ -1,6 +1,5 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function
+import typing
 
 import pkg_resources
 from flask import current_app, url_for
@@ -9,11 +8,16 @@ from webassets.filter import get_filter
 
 from abilian.services.security import Anonymous
 
-# register custom filters for webassets
-from . import filters  # noqa
+from .filters import register_filters
+from .mixin import AssetManagerMixin
+
+if typing.TYPE_CHECKING:
+    from abilian.app import Application
 
 
-def init_app(app):
+def init_app(app: "Application") -> None:
+    register_filters()
+
     assets = app.extensions["webassets"]
     assets.append_path(RESOURCES_DIR, "/static/abilian")
     app.add_static_url(
@@ -23,7 +27,7 @@ def init_app(app):
     app.before_first_request(requirejs_config)
 
 
-def requirejs_config():
+def requirejs_config() -> None:
     assets = current_app.extensions["webassets"]
     config = assets.requirejs_config
 
@@ -89,11 +93,11 @@ ABILIAN_JS = Bundle(
     "js/widgets/base.js",
     "js/widgets/select2.js",
     "js/widgets/richtext.js",
-    "js/widgets/delete.js",
     "js/widgets/file.js",
     "js/widgets/image.js",
     "js/widgets/tags.js",
     "js/widgets/dynamic-row.js",
+    "js/widgets/delete.js",
 )
 
 LESS = Bundle(
