@@ -1,19 +1,18 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any, Callable, Collection, Union, cast
 
 from flask import Blueprint as BaseBlueprint
-from six import string_types
 
+from abilian.core.models.subjects import User
 from abilian.services.security import Anonymous, Role
+from abilian.services.security.service import SecurityService
 
 
-def allow_anonymous(user, roles, **kwargs):
+def allow_anonymous(user: User, roles: Collection[Role], **kwargs: Any) -> bool:
     return True
 
 
-def allow_access_for_roles(roles):
+def allow_access_for_roles(roles: Union[Collection[Role], Role]) -> Callable:
     """Access control helper to check user's roles against a list of valid
     roles."""
     if isinstance(roles, Role):
@@ -23,10 +22,11 @@ def allow_access_for_roles(roles):
     if Anonymous in valid_roles:
         return allow_anonymous
 
-    def check_role(user, roles, **kwargs):
+    # FIXME: parameter not used. Why?
+    def check_role(user: User, roles: Collection[Role], **kwargs):
         from abilian.services import get_service
 
-        security = get_service("security")
+        security: SecurityService = cast(SecurityService, get_service("security"))
         return security.has_role(user, valid_roles)
 
     return check_role
@@ -36,7 +36,13 @@ class Blueprint(BaseBlueprint):
     """An enhanced :class:`flask.blueprints.Blueprint` with access control
     helpers."""
 
-    def __init__(self, name, import_name, allowed_roles=None, **kwargs):
+    def __init__(
+        self,
+        name: str,
+        import_name: str,
+        allowed_roles: Union[None, str, Role, Collection[Role]] = None,
+        **kwargs: Any
+    ) -> None:
         """
         :param allowed_roles: role or list of roles required to access any view in this
             blueprint.
@@ -44,7 +50,7 @@ class Blueprint(BaseBlueprint):
         BaseBlueprint.__init__(self, name, import_name, **kwargs)
 
         if allowed_roles is not None:
-            if isinstance(allowed_roles, string_types):
+            if isinstance(allowed_roles, str):
                 allowed_roles = Role(allowed_roles)
 
             if isinstance(allowed_roles, Role):
