@@ -1,34 +1,34 @@
-# coding: utf-8
 """Add a few specific filters to Jinja2."""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
-import datetime
 import re
 from calendar import timegm
+from datetime import date, datetime
 from functools import wraps
+from typing import Any, Callable, Optional, Union
 
 import bleach
 import dateutil.parser
 import flask_babel as babel
 from babel.dates import DateTimePattern, format_timedelta, parse_pattern
 from deprecated import deprecated
-from flask import Flask
-from jinja2 import Markup, escape, evalcontextfilter
+from jinja2 import Environment, escape, evalcontextfilter
+from jinja2.nodes import EvalContext
+from markupsafe import Markup
 from pytz import utc
-from six import text_type
 from werkzeug.routing import BuildError
 
-from ..core.util import local_dt, slugify, utc_dt
+from abilian.core.util import local_dt, slugify, utc_dt
+
 from .util import url_for
 
 
-def autoescape(filter_func):
+def autoescape(filter_func: Callable) -> Callable:
     """Decorator to autoescape result from filters."""
 
     @evalcontextfilter
     @wraps(filter_func)
-    def _autoescape(eval_ctx, *args, **kwargs):
+    def _autoescape(
+        eval_ctx: EvalContext, *args: str, **kwargs: Any
+    ) -> Union[Markup, str]:
         result = filter_func(*args, **kwargs)
         if eval_ctx.autoescape:
             result = Markup(result)
@@ -38,7 +38,7 @@ def autoescape(filter_func):
 
 
 @autoescape
-def nl2br(value):
+def nl2br(value: str) -> Markup:
     """Replace newlines with <br />."""
     result = escape(value).replace("\n", Markup("<br />\n"))
     return result
@@ -48,27 +48,25 @@ _PARAGRAPH_RE = re.compile(r"(?:\r\n|\r|\n){2,}")
 
 
 @autoescape
-def paragraphs(value):
+def paragraphs(value: str) -> str:
     """Blank lines delimitates paragraphs."""
     result = "\n\n".join(
-        (
-            "<p>{}</p>".format(p.strip().replace("\n", Markup("<br />\n")))
-            for p in _PARAGRAPH_RE.split(escape(value))
-        )
+        "<p>{}</p>".format(p.strip().replace("\n", Markup("<br />\n")))
+        for p in _PARAGRAPH_RE.split(escape(value))
     )
     return result
 
 
-def labelize(s):
+def labelize(s: str) -> str:
     return " ".join([w.capitalize() for w in s.split("_")])
 
 
-def filesize(d):
+def filesize(d: Union[int, str]) -> Markup:
     if not isinstance(d, int):
         d = int(d)
 
     if d < 1000:
-        s = "%d&nbsp;B" % d
+        s = f"{d:d}&nbsp;B"
 
     elif d < int(1e4):
         s = "%.1f&nbsp;kB" % (d / 1e3)
@@ -89,15 +87,15 @@ def filesize(d):
     return Markup(s)
 
 
-def roughsize(size, above=20, mod=10):
+def roughsize(size: int, above: int = 20, mod: int = 10) -> str:
     """6 -> '6' 15 -> '15' 134 -> '130+'."""
     if size < above:
-        return text_type(size)
+        return str(size)
 
-    return "{:d}+".format(size - size % mod)
+    return f"{(size - size % mod):d}+"
 
 
-def datetimeparse(s):
+def datetimeparse(s) -> Optional[datetime]:
     """Parse a string date time to a datetime object.
 
     Suitable for dates serialized with .isoformat()
@@ -112,11 +110,16 @@ def datetimeparse(s):
     return utc_dt(dt)
 
 
-def age(dt, now=None, add_direction=True, date_threshold=None):
+def age(
+    dt: Optional[datetime],
+    now: Optional[datetime] = None,
+    add_direction: bool = True,
+    date_threshold: Optional[Any] = None,
+) -> str:
     """
-    :param dt: :class:`datetime<datetime.datetime>` instance to format
+    :param dt: :class:`datetime<datetime>` instance to format
 
-    :param now: :class:`datetime<datetime.datetime>` instance to compare to `dt`
+    :param now: :class:`datetime<datetime>` instance to compare to `dt`
 
     :param add_direction: if `True`, will add "in" or "ago" (example for `en`
        locale) to time difference `dt - now`, i.e "in 9 min." or " 9min. ago"
@@ -129,7 +132,7 @@ def age(dt, now=None, add_direction=True, date_threshold=None):
         return ""
 
     if not now:
-        now = datetime.datetime.utcnow()
+        now = datetime.utcnow()
 
     locale = babel.get_locale()
     dt = utc_dt(dt)
@@ -166,27 +169,27 @@ def age(dt, now=None, add_direction=True, date_threshold=None):
     )
 
 
-def date_age(dt, now=None):
+def date_age(dt: Optional[datetime], now: Optional[datetime] = None) -> str:
     # Fail silently for now XXX
     if not dt:
         return ""
 
     formatted_date = babel.format_datetime(dt, format="yyyy-MM-dd HH:mm")
-    return "{} ({})".format(formatted_date, age(dt, now))
+    return f"{formatted_date} ({age(dt, now)})"
 
 
 @deprecated
-def date(value, format="EE, d MMMM y"):
+def date_fmt(value, format="EE, d MMMM y"):
     """
     @deprecated: use flask_babel's dateformat filter instead.
     """
-    if isinstance(value, datetime.date):
+    if isinstance(value, date):
         return babel.format_date(value, format)
     else:
         return babel.format_date(local_dt(value), format)
 
 
-def babel2datepicker(pattern):
+def babel2datepicker(pattern: DateTimePattern) -> str:
     """Convert date format from babel (http://babel.pocoo.org/docs/dates/#date-
     fields)) to a format understood by bootstrap-datepicker."""
     if not isinstance(pattern, DateTimePattern):
@@ -234,7 +237,7 @@ def to_timestamp(dt):
     return timegm(utc_datetime.timetuple()) + utc_datetime.microsecond / 1e6
 
 
-def abbrev(s, max_size):
+def abbrev(s: str, max_size: int) -> str:
     if len(s) <= max_size:
         return s
     else:
@@ -248,7 +251,7 @@ def bool2check(val, true="\u2713", false=""):
 
 
 @autoescape
-def linkify(s):
+def linkify(s: str) -> Markup:
     return Markup(bleach.linkify(s))
 
 
@@ -265,17 +268,13 @@ def obj_to_url(obj):
         return ""
 
 
-def init_filters(env):
-    if isinstance(env, Flask):
-        # old api for init_filters: we used to pass flask application
-        env = env.jinja_env
-
+def init_filters(env: Environment) -> None:
     env.filters["nl2br"] = nl2br
     env.filters["paragraphs"] = paragraphs
     env.filters["date_age"] = date_age
     env.filters["datetimeparse"] = datetimeparse
     env.filters["age"] = age
-    env.filters["date"] = date
+    env.filters["date"] = date_fmt
     env.filters["babel2datepicker"] = babel2datepicker
     env.filters["to_timestamp"] = to_timestamp
     env.filters["url_for"] = obj_to_url
