@@ -1,23 +1,21 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
-import datetime
 import logging
 import operator
+from datetime import datetime
 from functools import partial
+from typing import Any, List, Union
 
 import babel
 import babel.dates
 import sqlalchemy as sa
 import sqlalchemy.exc
+from babel.core import Locale
 from flask import current_app
 from flask.helpers import locked_cached_property
 from flask_babel import format_date, format_datetime, get_locale, get_timezone
 from flask_login import current_user
 from flask_wtf.file import FileField as BaseFileField
-from six import PY2, string_types
+from werkzeug.datastructures import ImmutableMultiDict
 from wtforms import Field
 from wtforms import FieldList as BaseFieldList
 from wtforms import FormField as BaseFormField
@@ -55,7 +53,7 @@ class FormField(BaseFormField):
     """Discard csrf_token on subform."""
 
     def process(self, *args, **kwargs):
-        super(FormField, self).process(*args, **kwargs)
+        super().process(*args, **kwargs)
         if isinstance(self.form, SecureForm):
             # don't create errors because of subtoken
             self._subform_csrf = self.form["csrf_token"]
@@ -78,7 +76,7 @@ class ModelFormField(FormField, BaseModelFormField):
     """Discard csrf_token on subform."""
 
 
-class FilterFieldListMixin(object):
+class FilterFieldListMixin:
     def validate(self, form, extra_validators=()):
         to_remove = []
         for field in self.entries:
@@ -99,7 +97,7 @@ class FilterFieldListMixin(object):
             # 3) by setting raw_data optional() does not reset the errors dict
             # -> subfields errors are propagated
             self.raw_data = [True]
-        return super(FilterFieldListMixin, self).validate(form, extra_validators)
+        return super().validate(form, extra_validators)
 
 
 class FieldList(FilterFieldListMixin, BaseFieldList):
@@ -110,7 +108,7 @@ class ModelFieldList(FilterFieldListMixin, BaseModelFieldList):
     """Filter empty entries before saving and refills before displaying."""
 
     def __init__(self, *args, **kwargs):
-        super(ModelFieldList, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         # build visible field list for widget. We must do it during form
         # instanciation so as to have permission filtering
@@ -138,7 +136,7 @@ class ModelFieldList(FilterFieldListMixin, BaseModelFieldList):
         """
         while len(self) < self.min_entries:
             self.append_entry()
-        return super(ModelFieldList, self).__call__(**kwargs)
+        return super().__call__(**kwargs)
 
 
 class FileField(BaseFileField):
@@ -156,7 +154,7 @@ class FileField(BaseFileField):
     blob = None
     blob_attr = "value"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         try:
             self.multiple = kwargs.pop("multiple")
         except KeyError:
@@ -184,7 +182,7 @@ class FileField(BaseFileField):
                 validators.append(DataRequired())
 
         kwargs["validators"] = validators
-        BaseFileField.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def allow_delete(self):
@@ -202,23 +200,23 @@ class FileField(BaseFileField):
     def has_file(self):
         return self._has_uploads
 
-    def process(self, formdata, *args, **kwargs):
-        delete_arg = "__{name}_delete__".format(name=self.name)
+    def process(self, formdata: ImmutableMultiDict, *args, **kwargs) -> None:
+        delete_arg = f"__{self.name}_delete__"
         self.delete_files_index = (
             formdata.getlist(delete_arg) if formdata and delete_arg in formdata else []
         )
 
-        return super(FileField, self).process(formdata, *args, **kwargs)
+        return super().process(formdata, *args, **kwargs)
 
-    def process_data(self, value):
+    def process_data(self, value: None) -> None:
         if isinstance(value, db.Model):
             self.blob = value
             value = getattr(value, self.blob_attr)
 
         self.object_data = value
-        return super(FileField, self).process_data(value)
+        return super().process_data(value)
 
-    def process_formdata(self, valuelist):
+    def process_formdata(self, valuelist: List[str]) -> None:
         uploads = current_app.extensions["uploads"]
         if self.delete_files_index:
             self.data = None
@@ -231,7 +229,7 @@ class FileField(BaseFileField):
 
             if fileobj is None:
                 # FIXME: this is a validation task
-                raise ValueError("File with handle {!r} not found".format(handle))
+                raise ValueError(f"File with handle {handle!r} not found")
 
             meta = uploads.get_metadata(current_user, handle)
             filename = meta.get("filename", handle)
@@ -258,7 +256,7 @@ class FileField(BaseFileField):
         mapper = state.mapper
         if name not in mapper.relationships:
             # directly store in database
-            return super(FileField, self).populate_obj(obj, name)
+            return super().populate_obj(obj, name)
 
         rel = getattr(mapper.relationships, name)
         if rel.uselist:
@@ -288,17 +286,23 @@ class FileField(BaseFileField):
 class DateTimeField(Field):
     widget = DateTimeInput()
 
-    def __init__(self, label=None, validators=None, use_naive=True, **kwargs):
+    def __init__(
+        self,
+        label: Union[str, None] = None,
+        validators: Any = None,
+        use_naive: bool = True,
+        **kwargs: Any,
+    ) -> None:
         """
         :param use_naive: if `False`, dates are considered entered using user's
         timezone; different users with different timezones will see corrected
         date/time. For storage dates are always stored using UTC.
         """
         self.raw_data = kwargs.pop("raw_data", None)
-        super(DateTimeField, self).__init__(label, validators, **kwargs)
+        super().__init__(label, validators, **kwargs)
         self.use_naive = use_naive
 
-    def _value(self):
+    def _value(self) -> str:
         if self.raw_data:
             return " ".join(self.raw_data)
         else:
@@ -313,13 +317,10 @@ class DateTimeField(Field):
                 .replace("y", "yyyy")
             )
             time_fmt = locale.time_formats["short"]
-            # Workaround bug in Babel (at least <= 2.4) under Python 3
-            if not PY2:
-                time_fmt = time_fmt.pattern
             dt_fmt = locale.datetime_formats["short"].format(time_fmt, date_fmt)
             return format_datetime(self.data, dt_fmt) if self.data else ""
 
-    def process_data(self, value):
+    def process_data(self, value: datetime) -> None:
         if value is not None:
             if not value.tzinfo:
                 if self.use_naive:
@@ -329,9 +330,9 @@ class DateTimeField(Field):
             if not self.use_naive:
                 value = value.astimezone(get_timezone())
 
-        super(DateTimeField, self).process_data(value)
+        super().process_data(value)
 
-    def process_formdata(self, valuelist):
+    def process_formdata(self, valuelist: List[str]) -> None:
         if valuelist:
             date_str = " ".join(valuelist)
             locale = get_locale()
@@ -342,9 +343,9 @@ class DateTimeField(Field):
             )  # force numerical months
             time_fmt = locale.time_formats["short"]
             time_fmt = babel2datetime(time_fmt)
-            datetime_fmt = "{} | {}".format(date_fmt, time_fmt)
+            datetime_fmt = f"{date_fmt} | {time_fmt}"
             try:
-                self.data = datetime.datetime.strptime(date_str, datetime_fmt)
+                self.data = datetime.strptime(date_str, datetime_fmt)
                 if not self.use_naive:
                     tz = get_timezone()
                     if self.data.tzinfo:
@@ -358,7 +359,7 @@ class DateTimeField(Field):
                 self.data = None
                 raise ValueError(self.gettext("Not a valid datetime value"))
 
-    def populate_obj(self, obj, name):
+    def populate_obj(self, obj: Any, name: str) -> None:
         dt = self.data
         if dt and self.use_naive:
             dt = dt.replace(tzinfo=None)
@@ -367,14 +368,16 @@ class DateTimeField(Field):
 
 
 class DateField(Field):
-    """A text field which stores a `datetime.date` matching a format."""
+    """A text field which stores a `date` matching a format."""
 
     widget = DateInput()
 
-    def __init__(self, label=None, validators=None, **kwargs):
-        super(DateField, self).__init__(label, validators, **kwargs)
+    def __init__(
+        self, label: Union[str, None] = None, validators: Any = None, **kwargs: Any
+    ) -> None:
+        super().__init__(label, validators, **kwargs)
 
-    def _value(self):
+    def _value(self) -> str:
         if self.raw_data:
             return " ".join(self.raw_data)
         else:
@@ -389,7 +392,7 @@ class DateField(Field):
             )
             return format_date(self.data, date_fmt) if self.data else ""
 
-    def process_formdata(self, valuelist):
+    def process_formdata(self, valuelist: List[str]) -> None:
         valuelist = [i for i in valuelist if i.strip()]
 
         if valuelist:
@@ -399,7 +402,7 @@ class DateField(Field):
             date_fmt = date_fmt.replace("%B", "%m").replace("%b", "%m")
 
             try:
-                strptime = datetime.datetime.strptime
+                strptime = datetime.strptime
                 self.data = strptime(date_str, date_fmt).date()
             except (ValueError, TypeError):
                 self.data = None
@@ -478,7 +481,7 @@ class QuerySelect2Field(SelectFieldBase):
         widget=None,
         multiple=False,
         collection_class=list,
-        **kwargs
+        **kwargs,
     ):
 
         if widget is None:
@@ -494,12 +497,12 @@ class QuerySelect2Field(SelectFieldBase):
         if not any(isinstance(v, (Optional, DataRequired)) for v in validators):
             logger = logging.getLogger(__name__ + "." + self.__class__.__name__)
             logger.warning(
-                'Use deprecated parameter `allow_blank` for field "{}".'.format(label)
+                f'Use deprecated parameter `allow_blank` for field "{label}".'
             )
             if not allow_blank:
                 validators.append(DataRequired())
 
-        super(QuerySelect2Field, self).__init__(label, validators, **kwargs)
+        super().__init__(label, validators, **kwargs)
 
         # PATCHED!
         if query_factory:
@@ -516,7 +519,7 @@ class QuerySelect2Field(SelectFieldBase):
 
         if get_label is None:
             self.get_label = lambda x: x
-        elif isinstance(get_label, string_types):
+        elif isinstance(get_label, str):
             self.get_label = operator.attrgetter(get_label)
         else:
             self.get_label = get_label
@@ -654,7 +657,7 @@ class JsonSelect2Field(SelectFieldBase):
         blank_text="",
         model_class=None,
         multiple=False,
-        **kwargs
+        **kwargs,
     ):
 
         self.multiple = multiple
@@ -663,7 +666,7 @@ class JsonSelect2Field(SelectFieldBase):
             widget = Select2Ajax(multiple=self.multiple)
 
         kwargs["widget"] = widget
-        super(JsonSelect2Field, self).__init__(label, validators, **kwargs)
+        super().__init__(label, validators, **kwargs)
         self.ajax_source = ajax_source
         self._model_class = model_class
 
@@ -736,7 +739,7 @@ class JsonSelect2Field(SelectFieldBase):
         try:
             state = sa.inspect(obj)
         except sa.exc.NoInspectionAvailable:
-            return super(JsonSelect2Field, self).populate_obj(obj, name)
+            return super().populate_obj(obj, name)
 
         relations = state.mapper.relationships
 
@@ -758,24 +761,22 @@ class JsonSelect2MultipleField(JsonSelect2Field):
 class LocaleSelectField(SelectField):
     widget = Select2()
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         kwargs["coerce"] = LocaleSelectField.coerce
-        kwargs["choices"] = [
-            locale_info for locale_info in i18n.supported_app_locales()
-        ]
-        super(LocaleSelectField, self).__init__(*args, **kwargs)
+        kwargs["choices"] = list(i18n.supported_app_locales())
+        super().__init__(*args, **kwargs)
 
     @staticmethod
-    def coerce(value):
+    def coerce(value: Locale) -> Locale:
         if isinstance(value, babel.Locale):
             return value
-        elif isinstance(value, string_types):
+        elif isinstance(value, str):
             return babel.Locale.parse(value)
         elif value is None:
             return None
 
         raise ValueError(
-            "Value cannot be converted to Locale(), or is not None, {!r}".format(value)
+            f"Value cannot be converted to Locale(), or is not None, {value!r}"
         )
 
     def iter_choices(self):
@@ -789,10 +790,10 @@ class LocaleSelectField(SelectField):
 class TimezoneField(SelectField):
     widget = Select2()
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         kwargs["coerce"] = babel.dates.get_timezone
-        kwargs["choices"] = [tz_info for tz_info in i18n.timezones_choices()]
-        super(TimezoneField, self).__init__(*args, **kwargs)
+        kwargs["choices"] = list(i18n.timezones_choices())
+        super().__init__(*args, **kwargs)
 
     def iter_choices(self):
         if not self.flags.required:
