@@ -1,21 +1,18 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import datetime
+from unittest import mock
 
-import mock
 import pytz
 from wtforms.form import Form
 
+from abilian.app import Application
 from abilian.core.entities import Entity
 from abilian.services.security import READ, WRITE, Anonymous, Owner, Role
 
 from . import FormPermissions, fields, filters
 
 
-def user_tz():
+def user_tz() -> str:
     # This one is GMT+8 and has no DST (tests should pass any time in year)
     return "Asia/Hong_Kong"
 
@@ -24,21 +21,21 @@ USER_TZ = pytz.timezone(user_tz())
 
 
 # test filters
-def test_strip():
+def test_strip() -> None:
     assert filters.strip(None) == ""
     assert filters.strip(4) == 4
     assert filters.strip(" a string ") == "a string"
     assert filters.strip(" voilà ") == "voilà"
 
 
-def test_uppercase():
+def test_uppercase() -> None:
     assert filters.uppercase(None) is None
     assert filters.uppercase(4) == 4
     assert filters.uppercase(" a string ") == " A STRING "
     assert filters.uppercase(" Voilà ") == " VOILÀ "
 
 
-def test_lowercase():
+def test_lowercase() -> None:
     assert filters.lowercase(None) is None
     assert filters.lowercase(4) == 4
     assert filters.lowercase(" A STRING ") == " a string "
@@ -46,7 +43,7 @@ def test_lowercase():
 
 
 # FormPermissions
-def test_form_permissions_controller():
+def test_form_permissions_controller() -> None:
     security_mock = mock.Mock()
     has_role = security_mock.has_role = mock.Mock()
     has_role.return_value = True
@@ -100,7 +97,7 @@ def test_form_permissions_controller():
         # field roles
         has_role.reset_mock()
         fp = FormPermissions(
-            default=MarkRole, read=Anonymous, fields_read={"test": Owner}
+            default=MarkRole, read=Anonymous, fields_read={"test": {Owner}}
         )
         fp.has_permission(READ)
         assert has_role.call_args[-1]["role"] == [Anonymous]
@@ -130,12 +127,12 @@ def test_form_permissions_controller():
         assert has_role.call_args[-1]["role"] == [Owner, MarkRole]
 
 
-def patch_babel(app):
+def patch_babel(app: Application) -> None:
     app.extensions["babel"].timezone_selector_func = None
     app.extensions["babel"].timezoneselector(user_tz)
 
 
-def test_datetime_field(app):
+def test_datetime_field(app: Application) -> None:
     """Test fields supports date with year < 1900."""
 
     assert "fr" in app.config["BABEL_ACCEPT_LANGUAGES"]
@@ -171,7 +168,7 @@ def test_datetime_field(app):
         assert field.data == expected_datetime
 
 
-def test_datetime_field_naive(app):
+def test_datetime_field_naive(app: Application) -> None:
     """Test fields supports date with year < 1900."""
     patch_babel(app)
 
@@ -195,7 +192,7 @@ def test_datetime_field_naive(app):
         assert obj.dt == datetime.datetime(1789, 6, 17, 10, 42)
 
 
-def test_datetime_field_force_4digit_year(app):
+def test_datetime_field_force_4digit_year(app: Application) -> None:
     # use 'en': short date pattern is 'M/d/yy'
     patch_babel(app)
 
@@ -206,7 +203,7 @@ def test_datetime_field_force_4digit_year(app):
         assert field._value() == "1/23/2011, 6:42 PM"
 
 
-def test_date_field(app):
+def test_date_field(app: Application) -> None:
     """Test fields supports date with year < 1900."""
     patch_babel(app)
 
@@ -218,7 +215,7 @@ def test_date_field(app):
         assert field._value() == "17/06/1789"
 
 
-def test_datefield_force_4digit_year(app):
+def test_datefield_force_4digit_year(app: Application) -> None:
     patch_babel(app)
 
     # use 'en': short date pattern is 'M/d/yy'
