@@ -1,12 +1,8 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 from babel.dates import DateTimePattern, parse_pattern
 
 
-def babel2datetime(pattern):
+def babel2datetime(pattern: DateTimePattern) -> str:
     """Convert date format from babel (http://babel.pocoo.org/docs/dates/#date-
     fields)) to a format understood by datetime.strptime."""
     if not isinstance(pattern, DateTimePattern):
