@@ -1,4 +1,3 @@
-# coding=utf-8
 """Validators.
 
 TODO: most of this is currently only stubs and needs to be implemented.
@@ -6,11 +5,9 @@ TODO: most of this is currently only stubs and needs to be implemented.
 NOTE: the `rule` property is supposed to be useful for generating client-side
 validation code.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Callable
 
-from wtforms import validators
-from wtforms.compat import string_types
+from wtforms import Field, Form, validators
 
 from abilian.i18n import _, _n
 from abilian.services import get_service
@@ -37,7 +34,7 @@ __all__ = (
 )
 
 
-class Rule(object):
+class Rule:
     @property
     def rule(self):
         return None
@@ -49,7 +46,7 @@ class Email(validators.Email):
             self.message = field.gettext("Invalid email address.")
 
         if field.data:
-            super(Email, self).__call__(form, field)
+            super().__call__(form, field)
 
     @property
     def rule(self):
@@ -57,10 +54,10 @@ class Email(validators.Email):
 
 
 class CorrectInputRequired(validators.DataRequired):
-    def __call__(self, form, field):
+    def __call__(self, form: Form, field: Field) -> None:
         if (
             field.data is None
-            or (isinstance(field.data, string_types) and not field.data.strip())
+            or (isinstance(field.data, str) and not field.data.strip())
             or (isinstance(field.data, (list, dict)) and not field.data)
         ):
             if self.message is None:
@@ -205,7 +202,7 @@ class AntiVirus(Rule):
             raise validators.ValidationError(_("Virus detected!"))
 
 
-class RenderEmpty(object):
+class RenderEmpty:
     """Force display."""
 
     field_flags = ("render_empty",)
@@ -218,7 +215,7 @@ class SIRET(RenderEmpty):
     pass
 
 
-def luhn(n):
+def luhn(n: str) -> bool:
     """Validate that a string made of numeric characters verify Luhn test. Used
     by siret validator.
 
@@ -239,11 +236,11 @@ def luhn(n):
 SIRET_CODES = ("001", "458", "462", "496", "372")
 
 
-def siret_validator():
+def siret_validator() -> Callable:
     """Validate a SIRET: check its length (14), its final code, and pass it
     through the Luhn algorithm."""
 
-    def _validate_siret(form, field, siret=""):
+    def _validate_siret(form: Form, field: Field, siret: str = "") -> None:
         """SIRET validator.
 
         A WTForm validator wants a form and a field as parameters. We
