@@ -1,30 +1,28 @@
-# coding=utf-8
 """Reusable widgets to be included in views.
 
 NOTE: code is currently quite messy. Needs to be refactored.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import base64
-import cgi
+import html
 import logging
 import re
 from collections import namedtuple
 from datetime import datetime
-from typing import Any, Dict, Optional, Text
+from typing import Any, Dict, List, Optional, Sequence, Text, Tuple, Union
+from urllib import parse
 
 import bleach
 import sqlalchemy as sa
 import sqlalchemy.orm
 import wtforms
-from flask import Markup, current_app, g, json, render_template, \
-    render_template_string
+from flask import current_app, g, json, render_template, render_template_string
 from flask_babel import format_date, format_datetime, format_number, get_locale
 from flask_login import current_user
 from flask_wtf.file import FileField
-from six import string_types, text_type
-from six.moves.urllib import parse
+from markupsafe import Markup
+from sqlalchemy.orm.mapper import Mapper
+from wtforms.fields import Field, IntegerField, Label, StringField
+from wtforms.form import Form
 from wtforms.widgets import HTMLString, Input
 from wtforms.widgets import PasswordInput as BasePasswordInput
 from wtforms.widgets import Select
@@ -48,6 +46,7 @@ __all__ = [
     "linkify_url",
     "text2html",
     "Column",
+    # Views
     "BaseTableView",
     "MainTableView",
     "RelatedTableView",
@@ -55,38 +54,40 @@ __all__ = [
     "SingleView",
     "Panel",
     "Row",
-    "Chosen",
-    "TagInput",
-    "DateInput",
-    "DefaultViewWidget",
+    # Widgets
     "BooleanWidget",
-    "FloatWidget",
+    "Chosen",
+    "DateInput",
+    "DateTimeInput",
     "DateTimeWidget",
     "DateWidget",
-    "MoneyWidget",
+    "DefaultViewWidget",
     "EmailWidget",
-    "URLWidget",
+    "EntityWidget",
+    "FileInput",
+    "FloatWidget",
+    "ImageInput",
     "ListWidget",
-    "TabularFieldListWidget",
     "ModelListWidget",
+    "MoneyWidget",
+    "RichTextWidget",
     "Select2",
     "Select2Ajax",
-    "RichTextWidget",
-    "FileInput",
-    "EntityWidget",
-    "ImageInput",
+    "TabularFieldListWidget",
+    "TagInput",
     "TextArea",
+    "URLWidget",
 ]
 
 
-def linkify_url(value):
+def linkify_url(value: str) -> str:
     """Tranform an URL pulled from the database to a safe HTML fragment."""
 
     value = value.strip()
 
     rjs = r"[\s]*(&#x.{1,7})?".join(list("javascript:"))
     rvb = r"[\s]*(&#x.{1,7})?".join(list("vbscript:"))
-    re_scripts = re.compile("({})|({})".format(rjs, rvb), re.IGNORECASE)
+    re_scripts = re.compile(f"({rjs})|({rvb})", re.IGNORECASE)
 
     value = re_scripts.sub("", value)
 
@@ -108,12 +109,10 @@ def linkify_url(value):
     if value.count("/") == 1 and value.endswith("/"):
         value = value[0:-1]
 
-    return '<a href="{}">{}</a>&nbsp;<i class="fa fa-external-link"></i>'.format(
-        url, value
-    )
+    return f'<a href="{url}">{value}</a>&nbsp;<i class="fa fa-external-link"></i>'
 
 
-def text2html(text):
+def text2html(text: str) -> Union[Markup, str]:
     text = text.strip()
     if re.search("<(p|br)>", text.lower()):
         return text
@@ -122,20 +121,22 @@ def text2html(text):
 
     lines = text.split("\n")
     lines = [line for line in lines if line]
-    paragraphs = ["<p>{}</p>".format(line) for line in lines]
+    paragraphs = [f"<p>{line}</p>" for line in lines]
     return Markup(bleach.clean("\n".join(paragraphs), tags=["p"]))
 
 
-class Column(object):
+class Column:
     def __init__(self, **kw):
         for k, w in kw.items():
             setattr(self, k, w)
 
 
-class View(object):
-    options = {}  # type: Dict[Text, Any]
+class View:
+    options: Dict[str, Any] = {}
 
-    def get_templates(self, specific_template_name, default_template):
+    def get_templates(
+        self, specific_template_name: str, default_template: str
+    ) -> List[str]:
         specific_template = self.options.get(specific_template_name)
         if specific_template:
             return [specific_template, default_template]
@@ -149,35 +150,39 @@ class BaseTableView(View):
     show_search = None
     paginate = False
 
-    def __init__(self, columns, options=None):
+    columns: List[Dict[str, Any]]
+
+    def __init__(
+        self, columns: List[Dict[str, Any]], options: Optional[Any] = None
+    ) -> None:
         if self.show_search is None:
             self.show_search = self.show_controls
 
+        self.columns = []
         self.init_columns(columns)
-        self.name = "{}-{:d}".format(
-            self.__class__.__name__.lower(), next(g.id_generator)
-        )
+        self.name = f"{self.__class__.__name__.lower()}-{next(g.id_generator):d}"
         if options is not None:
             self.options = options
             self.show_controls = self.options.get("show_controls", self.show_controls)
             self.show_search = self.options.get("show_search", self.show_controls)
             self.paginate = self.options.get("paginate", self.paginate)
 
-    def init_columns(self, columns):
+    def init_columns(self, columns: Sequence[Union[str, Dict[str, Any]]]) -> None:
         # TODO
-        self.columns = []
-        default_width = "{:2.0f}%".format(0.99 / len(columns) * 100)
+        default_width = f"{(0.99 / len(columns) * 100):2.0f}%"
         for col in columns:
-            if isinstance(col, string_types):
-                col = {"name": col, "width": default_width}
-            assert isinstance(col, dict)
-            col.setdefault("width", default_width)
-            col.setdefault("sorting", ("asc", "desc"))
-            if "label" not in col:
-                col["label"] = labelize(col["name"])
-            self.columns.append(col)
+            if isinstance(col, str):
+                column = {"name": col, "width": default_width}
+            else:
+                column = col
+            assert isinstance(column, dict)
+            column.setdefault("width", default_width)
+            column.setdefault("sorting", ("asc", "desc"))
+            if "label" not in column:
+                column["label"] = labelize(column["name"])
+            self.columns.append(column)
 
-    def render(self, entities, **kwargs):
+    def render(self, entities: List[Entity], **kwargs: Any) -> Markup:
         aoColumns = []
         aaSorting = []
         offset = 0
@@ -223,7 +228,7 @@ class BaseTableView(View):
             render_template(templates, table=table, js=Markup(js), view=self, **kwargs)
         )
 
-    def render_line(self, entity):
+    def render_line(self, entity: Entity) -> List[Union[Markup, str]]:
         line = []
         make_link_on = self.options.get("make_link_on")
 
@@ -250,18 +255,14 @@ class BaseTableView(View):
             if format:
                 cell = format(value)
             elif column_name in (make_link_on, "name") or col.get("linkable"):
-                cell = Markup(
-                    '<a href="{}">{}</a>'.format(
-                        build_url(entity), cgi.escape(text_type(value))
-                    )
-                )
+                url = build_url(entity)
+                text = html.escape(str(value))
+                cell = Markup(f'<a href="{url}">{text}</a>')
             elif isinstance(value, Entity):
-                cell = Markup(
-                    '<a href="{}">{}</a>'.format(
-                        build_url(value), cgi.escape(value.name)
-                    )
-                )
-            elif isinstance(value, string_types) and (
+                url = build_url(entity)
+                text = html.escape(value.name)
+                cell = Markup(f'<a href="{url}">{text}</a>')
+            elif isinstance(value, str) and (
                 value.startswith("http://") or value.startswith("www.")
             ):
                 cell = Markup(linkify_url(value))
@@ -270,11 +271,11 @@ class BaseTableView(View):
             elif isinstance(value, list):
                 cell = "; ".join(value)
             else:
-                if not isinstance(value, (Markup,) + string_types):
+                if not isinstance(value, (Markup,) + (str,)):
                     if value is None:
                         value = ""
                     else:
-                        value = text_type(value)
+                        value = str(value)
                 cell = value
 
             line.append(cell)
@@ -303,39 +304,47 @@ class AjaxMainTableView(View):
 
     show_controls = False
     paginate = True
+    columns: List[Dict[str, Any]]
+    options: Dict[str, Any]
 
     def __init__(
-        self, columns, ajax_source, search_criterions=(), name=None, options=None
-    ):
+        self,
+        columns: List[Dict[str, Any]],
+        ajax_source: str,
+        search_criterions: Tuple = (),
+        name: Optional[str] = None,
+        options: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        self.columns = []
         self.init_columns(columns)
         self.ajax_source = ajax_source
         self.search_criterions = search_criterions
         self.name = name if name is not None else id(self)
         self.save_state = name is not None
-        if options is not None:
-            self.options = options
+        self.options = options or {}
 
-    def init_columns(self, columns):
+    def init_columns(self, columns: Sequence[Union[str, Dict[str, Any]]]) -> None:
         # TODO: compute the correct width for each column.
-        self.columns = []
         default_width = 0.99 / len(columns)
         for col in columns:
             if isinstance(col, str):
-                col = {"name": col, "width": default_width}
-            assert isinstance(col, dict)
-            if "label" not in col:
-                col["label"] = labelize(col["name"])
-
-            col.setdefault("sorting", ["asc", "desc"])
-
-            if not col["sorting"]:
-                col.setdefault("sortable", False)
+                column = {"name": col, "width": default_width}
             else:
-                col.setdefault("sortable", True)
+                column = col
+            assert isinstance(column, dict)
+            if "label" not in column:
+                column["label"] = labelize(column["name"])
 
-            self.columns.append(col)
+            column.setdefault("sorting", ["asc", "desc"])
 
-    def render(self):
+            if not column["sorting"]:
+                column.setdefault("sortable", False)
+            else:
+                column.setdefault("sortable", True)
+
+            self.columns.append(column)
+
+    def render(self) -> Markup:
         aoColumns = [{"asSorting": []}] if self.show_controls else []
         aoColumns += [
             {"asSorting": col["sorting"], "bSortable": col["sortable"]}
@@ -374,7 +383,7 @@ class AjaxMainTableView(View):
                 continue
             d = {
                 "name": criterion.name,
-                "label": text_type(criterion.label),
+                "label": str(criterion.label),
                 "type": criterion.form_filter_type,
                 "args": criterion.form_filter_args,
                 "unset": criterion.form_unset_value,
@@ -422,25 +431,21 @@ class AjaxMainTableView(View):
             if has_custom_display:
                 cell = value
             elif column_name == "name":
-                cell = Markup(
-                    '<a href="{}">{}</a>'.format(url_for(entity), cgi.escape(value))
-                )
+                cell = Markup(f'<a href="{url_for(entity)}">{html.escape(value)}</a>')
             elif isinstance(value, Entity):
                 cell = Markup(
-                    '<a href="{}">{}</a>'.format(url_for(value), cgi.escape(value.name))
+                    f'<a href="{url_for(value)}">{html.escape(value.name)}</a>'
                 )
-            elif isinstance(value, string_types) and (
+            elif isinstance(value, str) and (
                 value.startswith("http://") or value.startswith("www.")
             ):
                 cell = Markup(linkify_url(value))
             elif col.get("linkable"):
                 cell = Markup(
-                    '<a href="{}">{}</a>'.format(
-                        url_for(entity), cgi.escape(text_type(value))
-                    )
+                    f'<a href="{url_for(entity)}">{html.escape(str(value))}</a>'
                 )
             else:
-                cell = text_type(value)
+                cell = str(value)
 
             line.append(cell)
         return line
@@ -452,12 +457,12 @@ class AjaxMainTableView(View):
 class SingleView(View):
     """View on a single object."""
 
-    def __init__(self, form, *panels, **options):
+    def __init__(self, form: Form, *panels: "Panel", **options: Any) -> None:
         self.form = form
         self.panels = panels
         self.options = options
 
-    def render(self, item, form, related_views=()):
+    def render(self, item: Entity, related_views: Tuple[()] = ()) -> Markup:
         mapper = sa.orm.class_mapper(item.__class__)
         panels = []
         _to_skip = (None, False, 0, 0.0, "", "-")
@@ -467,7 +472,7 @@ class SingleView(View):
             field_name_iter = (fn for row in panel.rows for fn in row)
 
             for name in field_name_iter:
-                field = form._fields[name]
+                field = self.form._fields[name]
                 if field.is_hidden:
                     continue
 
@@ -498,15 +503,15 @@ class SingleView(View):
             "related_views": related_views,
             "entity": item,
             "panels": panels,
-            "form": form,
+            "form": self.form,
         }
         return Markup(render_template(templates, **ctx))
 
-    def render_form(self, form, for_new=False, has_save_and_add_new=False):
+    def render_form(self, for_new=False, has_save_and_add_new=False):
         # Client-side rules for jQuery.validate
         # See: http://docs.jquery.com/Plugins/Validation/Methods#Validator
         rules = {}
-        for field in form:
+        for field in self.form:
             rules_for_field = {}
             for validator in field.validators:
                 rule_for_validator = getattr(validator, "rule", None)
@@ -522,14 +527,14 @@ class SingleView(View):
         templates = self.get_templates("edit_template", "widgets/render_for_edit.html")
         ctx = {
             "view": self,
-            "form": form,
+            "form": self.form,
             "for_new": for_new,
             "has_save_and_add_new": has_save_and_add_new,
             "rules": rules,
         }
         return Markup(render_template(templates, **ctx))
 
-    def label_for(self, field, mapper, name):
+    def label_for(self, field: Field, mapper: Mapper, name: str) -> Label:
         label = field.label
         if label is None:
             try:
@@ -551,7 +556,7 @@ class SingleView(View):
 #
 # Used to describe single entity views.
 #
-class Panel(object):
+class Panel:
     """`Panel` and `Row` classes help implement a trivial internal DSL for
     specifying multi-column layouts in forms or object views.
 
@@ -559,7 +564,7 @@ class Panel(object):
     designs eventually.
     """
 
-    def __init__(self, label=None, *rows):
+    def __init__(self, label: str, *rows: "Row") -> None:
         self.label = label
         self.rows = rows
 
@@ -573,7 +578,7 @@ class Panel(object):
         return len(self.rows)
 
 
-class Row(object):
+class Row:
     """`Panel` and `Row` classes help implement a trivial internal DSL for
     specifying multi-column layouts in forms or object views.
 
@@ -581,7 +586,7 @@ class Row(object):
     designs eventually.
     """
 
-    def __init__(self, *cols):
+    def __init__(self, *cols: str) -> None:
         self.cols = cols
 
     def __iter__(self):
@@ -594,7 +599,7 @@ class Row(object):
         return len(self.cols)
 
 
-class ModelWidget(object):
+class ModelWidget:
     edit_template = "widgets/model_widget_edit.html"
     view_template = "widgets/model_widget_view.html"
 
@@ -642,11 +647,18 @@ class TextInput(wtforms.widgets.TextInput):
     :class:`abilian.web.action.Icon`.
     """
 
-    pre_icon = None  # type: Optional[Text]
-    post_icon = None  # type: Optional[Text]
+    pre_icon: Optional[Text] = None
+    post_icon: Optional[Text] = None
 
-    def __init__(self, input_type=None, pre_icon=None, post_icon=None, *args, **kwargs):
-        super(TextInput, self).__init__(input_type, *args, **kwargs)
+    def __init__(
+        self,
+        input_type: Optional[Any] = None,
+        pre_icon: Optional[Any] = None,
+        post_icon: Optional[Any] = None,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__(input_type, *args, **kwargs)
 
         if pre_icon is not None:
             self.pre_icon = pre_icon
@@ -655,7 +667,7 @@ class TextInput(wtforms.widgets.TextInput):
 
     def __call__(self, field, *args, **kwargs):
         if not any((self.pre_icon, self.post_icon)):
-            return super(TextInput, self).__call__(field, *args, **kwargs)
+            return super().__call__(field, *args, **kwargs)
 
         kwargs.setdefault("type", self.input_type)
         if "value" not in kwargs:
@@ -693,8 +705,14 @@ class TextArea(BaseTextArea):
     resizeable = None
     rows = None
 
-    def __init__(self, resizeable=None, rows=None, *args, **kwargs):
-        BaseTextArea.__init__(self, *args, **kwargs)
+    def __init__(
+        self,
+        resizeable: Optional[str] = None,
+        rows: Optional[int] = None,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__(*args, **kwargs)
 
         if resizeable not in self._resizeable_valid:
             raise ValueError(
@@ -717,17 +735,17 @@ class TextArea(BaseTextArea):
         if self.rows and "rows" not in kwargs:
             kwargs["rows"] = self.rows
 
-        return super(TextArea, self).__call__(*args, **kwargs)
+        return super().__call__(*args, **kwargs)
 
 
-class FileInput(object):
+class FileInput:
     """Renders a file input.
 
     Inspired from http://www.abeautifulsite.net/blog/2013/08/whipping-
     file-inputs-into-shape-with-bootstrap-3/
     """
 
-    def __init__(self, template="widgets/file_input.html"):
+    def __init__(self, template: str = "widgets/file_input.html") -> None:
         self.template = template
 
     def __call__(self, field, **kwargs):
@@ -735,7 +753,7 @@ class FileInput(object):
         kwargs["name"] = field.name
         kwargs["type"] = "file"
         kwargs["disabled"] = "disabled"  # JS widget will activate it
-        input_elem = "<input {}>".format(html_params(**kwargs))
+        input_elem = f"<input {html_params(**kwargs)}>"
         button_label = _("Add file") if "multiple" in kwargs else _("Select file")
 
         existing = self.build_exisiting_files_list(field)
@@ -773,14 +791,22 @@ class FileInput(object):
 
         if not field.multiple:
             object_data = [object_data]
-            blob = getattr(field, "blob")
+            blob = field.blob
 
         for idx, data in enumerate(object_data):
             if data is not None:
+                if hasattr(data, "content_length"):
+                    size = data.content_length
+                else:
+                    try:
+                        size = len(bytes(data))
+                    except Exception:
+                        # FIXME: when does it happen ?
+                        size = 0
                 existing.append(
                     {
                         "file": data,
-                        "size": len(bytes(data)),
+                        "size": size,
                         "delete": idx in field.delete_files_index,
                     }
                 )
@@ -821,19 +847,19 @@ class ImageInput(FileInput):
 
     def __init__(
         self,
-        template="widgets/image_input.html",
-        width=120,
-        height=120,
-        resize_mode=image.CROP,
-        valid_extensions=("jpg", "jpeg", "png"),
-    ):
-        super(ImageInput, self).__init__(template=template)
+        template: str = "widgets/image_input.html",
+        width: int = 120,
+        height: int = 120,
+        resize_mode: str = image.CROP,
+        valid_extensions: Tuple[str, str, str] = ("jpg", "jpeg", "png"),
+    ) -> None:
+        super().__init__(template=template)
         self.resize_mode = resize_mode
         self.valid_extensions = valid_extensions
         self.width, self.height = width, height
 
     def build_exisiting_files_list(self, field):
-        existing = super(ImageInput, self).build_exisiting_files_list(field)
+        existing = super().build_exisiting_files_list(field)
         for data in existing:
             value = data["file"]
             if value:
@@ -849,7 +875,7 @@ class ImageInput(FileInput):
         return existing
 
     def build_uploads_list(self, field):
-        uploaded = super(ImageInput, self).build_uploads_list(field)
+        uploaded = super().build_uploads_list(field)
         for data in uploaded:
             value = data["file"]
             if value:
@@ -868,7 +894,7 @@ class ImageInput(FileInput):
     def get_thumb(self, data, width, height):
         try:
             get_format(data)
-        except IOError:
+        except OSError:
             return ""
         return image.resize(data, width, height, mode=self.resize_mode)
 
@@ -877,10 +903,10 @@ class ImageInput(FileInput):
             return ""
         try:
             fmt = image.get_format(img).lower()
-        except IOError:
+        except OSError:
             return ""
         thumb = base64.b64encode(img).decode("ascii")
-        return "data:image/{format};base64,{img}".format(format=fmt, img=thumb)
+        return f"data:image/{fmt};base64,{thumb}"
 
     def render_view(self, field, **kwargs):
         data = field.data
@@ -888,7 +914,7 @@ class ImageInput(FileInput):
             return ""
         try:
             get_format(data)
-        except IOError:
+        except OSError:
             return ""
 
         width = kwargs.get("width", self.width)
@@ -908,7 +934,7 @@ class Chosen(Select):
     def __call__(self, field, **kwargs):
         kwargs.setdefault("id", field.id)
         params = html_params(name=field.name, **kwargs)
-        html = ['<select {} class="chzn-select">'.format(params)]
+        html = [f'<select {params} class="chzn-select">']
         for val, label, selected in field.iter_choices():
             html.append(self.render_option(val, label, selected))
         html.append("</select>")
@@ -920,9 +946,7 @@ class Chosen(Select):
         if selected:
             options["selected"] = True
         params = html_params(**options)
-        return HTMLString(
-            "<option {}>{}</option>".format(params, cgi.escape(text_type(label)))
-        )
+        return HTMLString(f"<option {params}>{html.escape(str(label))}</option>")
 
 
 class TagInput(Input):
@@ -935,7 +959,7 @@ class TagInput(Input):
             kwargs["value"] = field._value()
 
         params = self.html_params(name=field.name, **kwargs)
-        return HTMLString("<input {}>".format(params))
+        return HTMLString(f"<input {params}>")
 
 
 class DateInput(Input):
@@ -979,7 +1003,7 @@ class DateInput(Input):
             "data-date-autoclose": "true",
         }
 
-        s = "<div {}>\n".format(html_params(**attributes))
+        s = f"<div {html_params(**attributes)}>\n"
 
         s += '  <input size="13" type="text" {} />\n'.format(
             html_params(name=field_name, id=field_id, value=value, **kwargs)
@@ -1002,17 +1026,17 @@ class TimeInput(Input):
 
     def __init__(
         self,
-        template=None,
-        widget_mode="dropdown",
-        h24_mode=True,
-        minuteStep=1,
-        showSeconds=False,
-        secondStep=1,
-        showInputs=False,
-        disableFocus=False,
-        modalBackdrop=False,
-    ):
-        Input.__init__(self)
+        template: None = None,
+        widget_mode: str = "dropdown",
+        h24_mode: bool = True,
+        minuteStep: int = 1,
+        showSeconds: bool = False,
+        secondStep: int = 1,
+        showInputs: bool = False,
+        disableFocus: bool = False,
+        modalBackdrop: bool = False,
+    ) -> None:
+        super().__init__()
 
         if template is not None:
             self.template = template
@@ -1062,11 +1086,11 @@ class TimeInput(Input):
         return Markup(render_template(self.template, **ctx))
 
 
-class DateTimeInput(object):
+class DateTimeInput:
     """if corresponding field.raw_data exist it is used to initialize default
     date & time (raw_data example: ["10/10/16 | 09:00"])"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.date = DateInput()
         self.time = TimeInput()
 
@@ -1134,14 +1158,16 @@ class DateTimeInput(object):
         )
 
 
-class DefaultViewWidget(object):
-    def render_view(self, field, **kwargs):
+class DefaultViewWidget:
+    def render_view(
+        self, field: Union[IntegerField, StringField], **kwargs: Any
+    ) -> str:
         value = field.object_data
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             return text2html(value)
         else:
             # [], None and other must be rendered using empty string
-            return text_type(value or "")
+            return str(value or "")
 
 
 class BooleanWidget(wtforms.widgets.CheckboxInput):
@@ -1163,8 +1189,9 @@ class BooleanWidget(wtforms.widgets.CheckboxInput):
             "wrapper-class",
         )
     )
+    on_off_options: Dict[str, Any]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.on_off_mode = kwargs.pop("on_off_mode", False)
         self.on_off_options = {}
         on_off_options = kwargs.pop("on_off_options", {})
@@ -1176,13 +1203,13 @@ class BooleanWidget(wtforms.widgets.CheckboxInput):
         if self.on_off_mode:
             self.on_off_options["data-toggle"] = "on-off"
 
-        super(BooleanWidget, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __call__(self, field, **kwargs):
         if self.on_off_mode:
             kwargs.update(self.on_off_options)
 
-        return super(BooleanWidget, self).__call__(field, **kwargs)
+        return super().__call__(field, **kwargs)
 
     def render_view(self, field, **kwargs):
         return "\u2713" if field.object_data else ""  # Text_type "Check mark"
@@ -1191,13 +1218,13 @@ class BooleanWidget(wtforms.widgets.CheckboxInput):
 class PasswordInput(BasePasswordInput):
     """Supports setting 'autocomplete' at instanciation time."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.autocomplete = kwargs.pop("autocomplete", None)
-        BasePasswordInput.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __call__(self, field, **kwargs):
         kwargs.setdefault("autocomplete", self.autocomplete)
-        return BasePasswordInput.__call__(self, field, **kwargs)
+        return super().__call__(field, **kwargs)
 
     def render_view(self, field, **kwargs):
         return "*****"
@@ -1209,7 +1236,7 @@ class FloatWidget(wtforms.widgets.TextInput):
     def __init__(self, precision=None):
         self.precision = precision
         if precision is not None:
-            self._fmt = ".{:d}f".format(precision)
+            self._fmt = f".{precision:d}f"
 
     def render_view(self, field, **kwargs):
         data = field.object_data
@@ -1229,15 +1256,13 @@ class DateTimeWidget(DateWidget):
         return format_datetime(field.object_data) if field.object_data else ""
 
 
-class EntityWidget(object):
+class EntityWidget:
     def render_view(self, field, **kwargs):
         objs = field.object_data
         if not field.multiple:
             objs = [objs]
         return ", ".join(
-            '<a href="{}">{}</a>'.format(url_for(o), cgi.escape(o.name))
-            for o in objs
-            if o
+            f'<a href="{url_for(o)}">{html.escape(o.name)}</a>' for o in objs if o
         )
 
 
@@ -1258,7 +1283,7 @@ class HoursWidget(TextInput):
             return ""
 
         # \u00A0: non-breakable whitespace
-        return "{value}\u00A0{unit}".format(value=val, unit=unit)
+        return f"{val}\u00A0{unit}"
 
 
 class MoneyWidget(TextInput):
@@ -1286,32 +1311,32 @@ class MoneyWidget(TextInput):
         # units, which we don't want
         #
         # \u00A0: non-breakable whitespace
-        return "{value}\u00A0{unit}".format(value=format_number(val), unit=unit)
+        return f"{format_number(val)} {unit}"
 
 
 class EmailWidget(TextInput):
     pre_icon = "@"
 
-    def render_view(self, field, **kwargs):
+    def render_view(self, field: StringField, **kwargs: Any) -> str:
         links = ""
         if isinstance(field, wtforms.fields.FieldList):
             for entry in field.entries:
                 link = bleach.linkify(entry.data, parse_email=True)
                 if link:
-                    links += ' {}&nbsp;<i class="fa fa-envelope"></i><br>'.format(link)
+                    links += f' {link}&nbsp;<i class="fa fa-envelope"></i><br>'
         else:
             link = bleach.linkify(field.object_data, parse_email=True)
             if link:
-                links = '{}&nbsp;<i class="fa fa-envelope"></i>'.format(link)
+                links = f'{link}&nbsp;<i class="fa fa-envelope"></i>'
         return links
 
 
-class URLWidget(object):
+class URLWidget:
     def render_view(self, field, **kwargs):
         return linkify_url(field.object_data) if field.object_data else ""
 
 
-class RichTextWidget(object):
+class RichTextWidget:
     template = "widgets/richtext.html"
     allowed_tags = {
         "a": {"href": True, "title": True},
@@ -1357,20 +1382,22 @@ class RichTextWidget(object):
 class ListWidget(wtforms.widgets.ListWidget):
     """display field label is optionnal."""
 
-    def __init__(self, html_tag="ul", prefix_label=True, show_label=True):
-        wtforms.widgets.ListWidget.__init__(self, html_tag, prefix_label)
+    def __init__(
+        self, html_tag: str = "ul", prefix_label: bool = True, show_label: bool = True
+    ) -> None:
+        super().__init__(html_tag, prefix_label)
         self.show_label = show_label
 
     def __call__(self, field, **kwargs):
         if self.show_label:
-            return super(ListWidget, self).__call__(field, **kwargs)
+            return super().__call__(field, **kwargs)
 
         kwargs.setdefault("id", field.id)
-        html = ["<{} {}>".format(self.html_tag, wtforms.widgets.html_params(**kwargs))]
+        html = [f"<{self.html_tag} {wtforms.widgets.html_params(**kwargs)}>"]
         for subfield in field:
-            html.append("<li>{}</li>".format(subfield()))
+            html.append(f"<li>{subfield()}</li>")
 
-        html.append("</{}>".format(self.html_tag))
+        html.append(f"</{self.html_tag}>")
         return wtforms.widgets.HTMLString("".join(html))
 
     def render_view(self, field, **kwargs):
@@ -1392,7 +1419,7 @@ class ListWidget(wtforms.widgets.ListWidget):
         return Markup(render_template_string(tpl, data=data))
 
 
-class FieldListWidget(object):
+class FieldListWidget:
     """For list of Fields (using <tr><td>)"""
 
     view_template = "widgets/fieldlist_view.html"
@@ -1416,7 +1443,7 @@ class FieldListWidget(object):
         return Markup(render_template(self.view_template, field=field))
 
 
-class TabularFieldListWidget(object):
+class TabularFieldListWidget:
     """For list of formfields.
 
     2 templates are available:
@@ -1444,7 +1471,7 @@ class TabularFieldListWidget(object):
         return Markup(render_template(self.template, labels=labels, field=field))
 
 
-class ModelListWidget(object):
+class ModelListWidget:
     def __init__(self, template="widgets/horizontal_table.html"):
         self.template = template
 
@@ -1487,8 +1514,14 @@ class Select2(Select):
 
     unescape_html = False
 
-    def __init__(self, unescape_html=False, js_init="select2", *args, **kwargs):
-        super(Select2, self).__init__(*args, **kwargs)
+    def __init__(
+        self,
+        unescape_html: bool = False,
+        js_init: str = "select2",
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__(*args, **kwargs)
         self.js_init = js_init
 
         if unescape_html:
@@ -1512,9 +1545,7 @@ class Select2(Select):
         return Select.__call__(self, field, *args, **kwargs)
 
     def render_view(self, field, **kwargs):
-        labels = [
-            text_type(label) for v, label, checked in field.iter_choices() if checked
-        ]
+        labels = [str(label) for v, label, checked in field.iter_choices() if checked]
         return "; ".join(labels)
 
     @classmethod
@@ -1524,7 +1555,7 @@ class Select2(Select):
         return Select.render_option(value, label, selected, **kwargs)
 
 
-class Select2Ajax(object):
+class Select2Ajax:
     """Ad-hoc select widget based on Select2.
 
     The code below is probably very fragile, since it depends on the internal
@@ -1588,9 +1619,9 @@ class Select2Ajax(object):
         s2_params.update(self.s2_params)
 
         if field.ajax_source:
-            s2_params["ajax"] = {"url": text_type(field.ajax_source)}
+            s2_params["ajax"] = {"url": str(field.ajax_source)}
 
-        s2_params["placeholder"] = text_type(field.label.text)
+        s2_params["placeholder"] = str(field.label.text)
 
         if not field.flags.required:
             s2_params["allowClear"] = True
@@ -1600,7 +1631,7 @@ class Select2Ajax(object):
             data = [data]
 
         values = self.values_builder(data)
-        input_value = ",".join(text_type(o.id) for o in data if o)
+        input_value = ",".join(str(o.id) for o in data if o)
         data_node_id = None
 
         json_data = {}
