@@ -1,28 +1,26 @@
-# coding=utf-8
 """Front-end for a CRM app.
 
 This should eventually allow implementing very custom CRM-style
 application.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import copy
 import logging
 import re
 from collections import OrderedDict
-from typing import List, Tuple
+from typing import Any, Callable, Collection, Dict, List, Optional, Tuple
 
 import sqlalchemy as sa
-from flask import Blueprint, current_app, g, jsonify, redirect, \
-    render_template, request, session, url_for
+from flask import Request, current_app, g, redirect, render_template, \
+    request, session, url_for
+from flask.blueprints import Blueprint, BlueprintSetupState
 from flask_login import current_user
-from six import add_metaclass
 from sqlalchemy import Date, DateTime, func, orm
 from sqlalchemy.sql.expression import asc, desc, nullsfirst, nullslast
 from werkzeug.exceptions import BadRequest
+from wtforms import Form
 
-from abilian.core.entities import Entity
+from abilian.app import Application
+from abilian.core.entities import Entity, EntityQuery
 from abilian.core.extensions import db
 from abilian.i18n import _l
 from abilian.services import audit_service, get_service
@@ -48,13 +46,13 @@ class ModuleAction(Action):
     `'module:{module.endpoint}'`
     """
 
-    def __init__(self, module, group, name, *args, **kwargs):
+    def __init__(
+        self, module: "Module", group: str, name: str, *args, **kwargs
+    ) -> None:
         self.group = group
-        super(ModuleAction, self).__init__(
-            module.action_category, name, *args, **kwargs
-        )
+        super().__init__(module.action_category, name, *args, **kwargs)
 
-    def pre_condition(self, context):
+    def pre_condition(self, context: Dict[str, "Module"]) -> bool:
         module = actions.context.get("module")
         if not module:
             return False
@@ -96,7 +94,7 @@ def add_to_recent_items(entity, type="ignored"):
     session["recent_items"] = g.recent_items = new_recent_items
 
 
-def expose(url="/", methods=("GET",)):
+def expose(url: str = "/", methods: Tuple[str] = ("GET",)) -> Callable:
     """Use this decorator to expose views in your view classes.
 
     `url`   Relative URL for the view `methods`   Allowed HTTP methods.
@@ -112,11 +110,11 @@ def expose(url="/", methods=("GET",)):
     return wrap
 
 
-def labelize(s):
+def labelize(s: str) -> str:
     return " ".join([w.capitalize() for w in s.split("_")])
 
 
-def make_single_view(form, **options):
+def make_single_view(form: Form, **options) -> SingleView:
     panels = []
     for gr in form._groups.items():
         panel = Panel(gr[0], *[Row(x) for x in gr[1]])
@@ -124,25 +122,27 @@ def make_single_view(form, **options):
     return SingleView(form, *panels, **options)
 
 
-class ModuleView(object):
+class ModuleView:
     """Mixin for module base views.
 
     Provide :attr:`module`.
     """
 
     #: :class:`Module` instance
-    module = None  # type: Module
+    module: "Module"
 
-    def __init__(self, module, *args, **kwargs):
+    def __init__(self, module: "Module", *args, **kwargs) -> None:
         self.module = module
-        super(ModuleView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class BaseEntityView(ModuleView):
     pk = "entity_id"
 
+    form: Form
+
     def init_object(self, args, kwargs):
-        args, kwargs = super(BaseEntityView, self).init_object(args, kwargs)
+        args, kwargs = super().init_object(args, kwargs)
 
         # security check
         if self.obj and not self.check_access():
@@ -159,7 +159,7 @@ class BaseEntityView(ModuleView):
         )
 
     def prepare_args(self, args, kwargs):
-        args, kwargs = super(BaseEntityView, self).prepare_args(args, kwargs)
+        args, kwargs = super().prepare_args(args, kwargs)
         actions.context["module"] = self.module
         add_to_recent_items(self.obj)
         return args, kwargs
@@ -168,13 +168,13 @@ class BaseEntityView(ModuleView):
         return redirect(self.module.url)
 
     @property
-    def single_view(self):
+    def single_view(self) -> SingleView:
         return make_single_view(
             self.form,
             view_template=self.module.view_template,
             view=self,
             module=self.module,
-            **self.module.view_options
+            **self.module.view_options,
         )
 
     def check_access(self):
@@ -247,9 +247,7 @@ class EntityView(BaseEntityView, ObjectView):
     def template_kwargs(self):
         module = self.module
         related_views = [v.render(self.obj) for v in module.related_views]
-        rendered_entity = self.single_view.render(
-            self.obj, self.form, related_views=related_views
-        )
+        rendered_entity = self.single_view.render(self.obj, related_views=related_views)
         audit_entries = audit_service.entries_for(self.obj)
 
         return {
@@ -268,7 +266,7 @@ class EntityEdit(BaseEntityView, ObjectEdit):
 
     @property
     def template_kwargs(self):
-        rendered_entity = self.single_view.render_form(self.form)
+        rendered_entity = self.single_view.render_form()
         return {
             "rendered_entity": rendered_entity,
             "show_new_comment_form": False,
@@ -289,7 +287,7 @@ class EntityCreate(BaseEntityView, ObjectCreate):
 
     @property
     def template_kwargs(self):
-        rendered_entity = self.single_view.render_form(self.form)
+        rendered_entity = self.single_view.render_form()
         return {
             "rendered_entity": rendered_entity,
             "for_new": True,
@@ -304,7 +302,7 @@ class EntityDelete(BaseEntityView, ObjectDelete):
 class ListJson(ModuleView, JSONView):
     """JSON endpoint, for AJAX-backed table views."""
 
-    def data(self, *args, **kwargs):
+    def data(self, *args, **kwargs) -> Dict:
         echo = int(kwargs.get("sEcho", 0))
         length = int(kwargs.get("iDisplayLength", 10))
         start = int(kwargs.get("iDisplayStart", 0))
@@ -323,13 +321,12 @@ class ListJson(ModuleView, JSONView):
         )
 
         data = [table_view.render_line(e) for e in entities]
-        result = {
+        return {
             "sEcho": echo,
             "iTotalRecords": total_count,
             "iTotalDisplayRecords": count,
             "aaData": data,
         }
-        return result
 
 
 class ModuleMeta(type):
@@ -339,7 +336,7 @@ class ModuleMeta(type):
     the class) to avoid calculating them for each view class instance.
     """
 
-    def __init__(cls, classname, bases, fields):
+    def __init__(cls, classname: str, bases: Tuple, fields: Dict[str, Any]) -> None:
         type.__init__(cls, classname, bases, fields)
 
         # Gather exposed views
@@ -361,10 +358,10 @@ class ModuleMeta(type):
                         # setattr(cls, p, _wrap_view(attr))
 
 
-class ModuleComponent(object):
+class ModuleComponent:
     """A component that provide new functions for a :class:`Module`"""
 
-    name = None  # type: str
+    name: str = None
 
     def __init__(self, name=None):
         if name is not None:
@@ -379,23 +376,20 @@ class ModuleComponent(object):
 
     def init(self, *args, **kwargs):
         """Implements this in components."""
-        pass
 
     def get_actions(self):
         return []
 
 
-@add_metaclass(ModuleMeta)
-class Module(object):
-
-    id = None  # type: str
-    endpoint = None  # type: str
-    label = None  # type: str
-    managed_class = None  # type: type
+class Module(metaclass=ModuleMeta):
+    id: str = None
+    endpoint: str = None
+    label: str = None
+    managed_class: type = None
     list_view = None
-    list_view_columns = []  # type: List
+    list_view_columns: List[Dict[str, Any]] = []
     single_view = None
-    components = ()  # type: Tuple
+    components: Tuple = ()
 
     # class based views. If not provided will be automaticaly created from
     # EntityView etc defined below
@@ -417,16 +411,16 @@ class Module(object):
     static_folder = None
     view_template = None
     view_options = None
-    related_views = []  # type: List[RelatedView]
+    related_views: List["RelatedView"] = []
     blueprint = None
     search_criterions = (
         search.TextSearchCriterion("name", attributes=("name", "nom")),
     )
     # used mostly to change datatable search_label
     tableview_options = {}  # type: ignore
-    _urls = []  # type: List[Tuple]
+    _urls: List[Tuple] = []
 
-    def __init__(self):
+    def __init__(self) -> None:
         # If endpoint name is not provided, get it from the class name
         if self.endpoint is None:
             class_name = self.__class__.__name__
@@ -465,7 +459,7 @@ class Module(object):
             "entity_view",
             self.view_cls,
             Form=self.view_form_class,
-            **kw
+            **kw,
         )
         view_endpoint = self.endpoint + ".entity_view"
 
@@ -475,7 +469,7 @@ class Module(object):
             self.edit_cls,
             Form=self.edit_form_class,
             view_endpoint=view_endpoint,
-            **kw
+            **kw,
         )
 
         self._setup_view(
@@ -485,7 +479,7 @@ class Module(object):
             Form=self.edit_form_class,
             chain_create_allowed=self.view_new_save_and_add,
             view_endpoint=view_endpoint,
-            **kw
+            **kw,
         )
 
         self._setup_view(
@@ -494,7 +488,7 @@ class Module(object):
             self.delete_cls,
             Form=self.edit_form_class,
             view_endpoint=view_endpoint,
-            **kw
+            **kw,
         )
 
         self._setup_view("/json", "list_json", ListJson, module=self)
@@ -523,13 +517,13 @@ class Module(object):
     def get_component(self, name):
         return self.__components.get(name)
 
-    def _setup_view(self, url, attr, cls, *args, **kwargs):
+    def _setup_view(self, url: str, attr: str, cls: Any, *args, **kwargs) -> None:
         """Register class based views."""
         view = cls.as_view(attr, *args, **kwargs)
         setattr(self, attr, view)
         self._urls.append((url, attr, view.methods))
 
-    def init_related_views(self):
+    def init_related_views(self) -> None:
         related_views = []
         for view in self.related_views:
             if not isinstance(view, RelatedView):
@@ -538,10 +532,10 @@ class Module(object):
         self.related_views = related_views
 
     @property
-    def action_category(self):
-        return "module:{}".format(self.endpoint)
+    def action_category(self) -> str:
+        return f"module:{self.endpoint}"
 
-    def get_grouped_actions(self):
+    def get_grouped_actions(self) -> OrderedDict:
         items = actions.for_category(self.action_category)
         groups = OrderedDict()
         for action in items:
@@ -549,7 +543,7 @@ class Module(object):
 
         return groups
 
-    def register_actions(self):
+    def register_actions(self) -> None:
         ACTIONS = [
             ModuleAction(
                 self,
@@ -566,7 +560,7 @@ class Module(object):
 
         actions.register(*ACTIONS)
 
-    def create_blueprint(self, crud_app):
+    def create_blueprint(self, crud_app: "CRUDApp") -> Blueprint:
         """Create a Flask blueprint for this module."""
         # Store admin instance
         self.crud_app = crud_app
@@ -574,10 +568,10 @@ class Module(object):
 
         # If url is not provided, generate it from endpoint name
         if self.url is None:
-            self.url = "{}/{}".format(self.crud_app.url, self.endpoint)
+            self.url = f"{self.crud_app.url}/{self.endpoint}"
         else:
             if not self.url.startswith("/"):
-                self.url = "{}/{}".format(self.crud_app.url, self.url)
+                self.url = f"{self.crud_app.url}/{self.url}"
 
         # Create blueprint and register rules
         self.blueprint = Blueprint(self.endpoint, __name__, url_prefix=self.url)
@@ -596,16 +590,16 @@ class Module(object):
 
         return self.blueprint
 
-    def _setup_breadcrumb_preprocessors(self, state):
+    def _setup_breadcrumb_preprocessors(self, state: BlueprintSetupState) -> None:
         self.blueprint.url_value_preprocessor(self._add_breadcrumb)
 
-    def _add_breadcrumb(self, endpoint, values):
+    def _add_breadcrumb(self, endpoint: str, values: Dict[Any, Any]) -> None:
         g.breadcrumb.append(
             BreadcrumbItem(label=self.label, url=Endpoint(".list_view"))
         )
 
     @property
-    def base_query(self):
+    def base_query(self) -> EntityQuery:
         """Return a query instance for :attr:`managed_class`."""
         return self.managed_class.query
 
@@ -616,7 +610,7 @@ class Module(object):
         return self.base_query.with_permission(READ)
 
     @property
-    def listing_query(self):
+    def listing_query(self) -> EntityQuery:
         """Like `read_query`, but can be made lightweight with only columns and
         joins of interest.
 
@@ -625,7 +619,7 @@ class Module(object):
         """
         return self.base_query.with_permission(READ)
 
-    def query(self, request):
+    def query(self, request: Request):
         """Return filtered query based on request args."""
         args = request.args
         search = args.get("sSearch", "").replace("%", "").lower()
@@ -636,7 +630,7 @@ class Module(object):
 
         return query
 
-    def list_query(self, request):
+    def list_query(self, request: Request) -> EntityQuery:
         """Return a filtered query based on request args, for listings.
 
         Like `query`, but subclasses can modify it to remove costly
@@ -652,7 +646,9 @@ class Module(object):
 
         return query
 
-    def ordered_query(self, request, query=None):
+    def ordered_query(
+        self, request: Request, query: Optional[EntityQuery] = None
+    ) -> EntityQuery:
         """Order query according to request args.
 
         If query is None, the query is generated according to request
@@ -703,7 +699,7 @@ class Module(object):
                 try:
                     direction = desc if sort_dir == "desc" else asc
                     sort_col = direction(sort_col)
-                except BaseException:
+                except Exception:
                     # FIXME
                     pass
 
@@ -713,7 +709,7 @@ class Module(object):
                     nullsorder = nullslast if sort_dir == "desc" else nullsfirst
                     try:
                         sort_col = nullsorder(sort_col)
-                    except BaseException:
+                    except Exception:
                         # FIXME
                         pass
 
@@ -722,7 +718,7 @@ class Module(object):
         if sort_cols:
             try:
                 query = query.order_by(*sort_cols)
-            except BaseException:
+            except Exception:
                 # FIXME
                 pass
         query.reset_joinpoint()
@@ -732,7 +728,7 @@ class Module(object):
     # Exposed views
     #
     @expose("/")
-    def list_view(self):
+    def list_view(self) -> str:
         actions.context["module"] = self
         table_view = AjaxMainTableView(
             name=self.managed_class.__name__.lower(),
@@ -786,8 +782,7 @@ class Module(object):
             raise BadRequest()
 
         results = self.list_json2_query_all(q)
-        results = {"results": results}
-        return jsonify(results)
+        return {"results": results}
 
     #
     # Utils
@@ -796,7 +791,7 @@ class Module(object):
         return request.path.startswith(self.url)
 
     @staticmethod
-    def _prettify_name(name):
+    def _prettify_name(name: str) -> str:
         """Prettify class name by splitting name by capital characters.
 
         So, 'MySuperClass' will look like 'My Super Class'
@@ -807,7 +802,7 @@ class Module(object):
         return re.sub(r"(?<=.)([A-Z])", r" \1", name)
 
 
-class RelatedView(object):
+class RelatedView:
     """A base class for related views."""
 
     def render(self, entity):
@@ -842,11 +837,13 @@ class DefaultRelatedView(RelatedView):
 
 
 # TODO: rename to CRMApp ?
-class CRUDApp(object):
+class CRUDApp:
 
-    modules = ()
+    modules: Collection[Module]
 
-    def __init__(self, app, modules=None, name=None):
+    def __init__(
+        self, app: Application, modules: None = None, name: None = None
+    ) -> None:
         if name is None:
             name = self.__class__.__module__
             modules_signature = ",".join(str(module.id) for module in self.modules)
@@ -869,9 +866,9 @@ class CRUDApp(object):
 
         return None
 
-    def add_module(self, module):
+    def add_module(self, module: Module) -> None:
         self.app.register_blueprint(self.create_blueprint(module))
         module.register_actions()
 
-    def create_blueprint(self, module):
+    def create_blueprint(self, module: Module) -> Blueprint:
         return module.create_blueprint(self)
