@@ -542,7 +542,15 @@ require(["AbilianNS", "jquery", "jquery.dataTables"], function(Abilian, $) {
 
       for (var i = 0; i < options.length; i++) {
         var opt = options[i];
-        s2_options.push({ id: opt[0], text: opt[1] });
+        if (opt[0]) {
+          s2_options.push({ id: opt[0], text: opt[1] });
+        } else if (opt.label) {
+          s2_options.push({ id: opt.id, text: opt.label });
+        } else if (opt.text) {
+          s2_options.push({ id: opt.id, text: opt.text });
+        } else {
+          // Do nothing.
+        }
       }
 
       $select.select2({
@@ -670,7 +678,7 @@ require(["AbilianNS", "jquery", "jquery.dataTables"], function(Abilian, $) {
       this.name = name;
       this.label = label;
       this.$elements = $('<div class="form-inline">');
-      this.multiple = s2_args["multiple"] || false;
+      this.multiple = s2_args.multiple || false;
 
       /* create the select */
       var selectId = name + "-select";
