@@ -1,8 +1,4 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import logging
 from collections import OrderedDict
 from functools import partial
@@ -12,7 +8,6 @@ import whoosh.highlight
 import whoosh.searching
 import whoosh.sorting
 from flask import Blueprint, current_app, g, render_template, request, url_for
-from six import text_type
 
 from abilian.i18n import _
 from abilian.services import get_service
@@ -49,14 +44,12 @@ def init_search(endpoint, values):
     q = request.args.get("q", "")
     try:
         page = max(int(request.args.get("page", 1)), 1)
-    except BaseException:
+    except Exception:
         page = 1
 
     g.breadcrumb.append(
         BreadcrumbItem(
-            label='"{}"'.format(q),
-            icon="search",
-            url=Endpoint("search.search_main", q=q),
+            label=f'"{q}"', icon="search", url=Endpoint("search.search_main", q=q)
         )
     )
 
@@ -75,7 +68,7 @@ def init_search(endpoint, values):
     if page > 1:
         g.breadcrumb.append(
             BreadcrumbItem(
-                label=text_type(page),
+                label=str(page),
                 url=Endpoint("search.search_main", page=page, **page_kw),
             )
         )
@@ -185,7 +178,6 @@ class Live(views.JSONView):
         svc = get_service("indexing")
         url_for_hit = svc.app_state.url_for_hit
         search_kwargs = {"facet_by_type": 5}
-        response = {}
         results = svc.search(q, **search_kwargs)
         datasets = {}
 
@@ -205,8 +197,7 @@ class Live(views.JSONView):
                     dataset.append(d)
             datasets[typename] = dataset
 
-        response["results"] = datasets
-        return response
+        return {"results": datasets}
 
 
 route("/live")(Live.as_view("live"))
