@@ -1,6 +1,5 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function
+from flask import Flask
 
 from .criterion import TagCriterion
 from .extension import TagsExtension
@@ -8,5 +7,5 @@ from .extension import TagsExtension
 __all__ = ["TagsExtension", "TagCriterion"]
 
 
-def register_plugin(app):
-    TagsExtension(app=app)
+def register_plugin(app: Flask) -> None:
+    TagsExtension(app)
