@@ -1,15 +1,10 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
-
 import sys
-from typing import List, Text
+from typing import List
 
 from blinker import Namespace, Signal
 from flask import current_app, g
 from flask_debugtoolbar.panels import DebugPanel
-from six import text_type
 
 from abilian.web.action import actions
 
@@ -29,8 +24,7 @@ class ActionDebugPanel(DebugPanel):
     def url(self):
         return ""
 
-    def content(self):
-        # type: () -> Text
+    def content(self) -> str:
         actions_for_template = []
 
         for category in actions.actions().keys():
@@ -42,12 +36,12 @@ class ActionDebugPanel(DebugPanel):
                     "class": action.__class__.__name__,
                 }
                 try:
-                    d["endpoint"] = text_type(action.endpoint)
-                except BaseException:
+                    d["endpoint"] = str(action.endpoint)
+                except Exception:
                     d["endpoint"] = "<Exception>"
                 try:
-                    d["url"] = text_type(action.url(g.action_context))
-                except BaseException:
+                    d["url"] = str(action.url(g.action_context))
+                except Exception:
                     d["url"] = "<Exception>"
                 actions_for_template.append(d)
 
@@ -78,8 +72,7 @@ class SignalsDebugPanel(DebugPanel):
     def url(self):
         return ""
 
-    def content(self):
-        # type: () -> Text
+    def content(self) -> str:
         module_names = sorted(sys.modules.keys())
 
         signals = []
@@ -96,7 +89,7 @@ class SignalsDebugPanel(DebugPanel):
                 ns = var
                 ns_name = var_name
                 for signal_name, signal in ns.items():
-                    receivers = [text_type(r) for r in signal.receivers.values()]
+                    receivers = [str(r) for r in signal.receivers.values()]
                     d = {
                         "module_name": module_name,
                         "ns_name": ns_name,
@@ -106,7 +99,7 @@ class SignalsDebugPanel(DebugPanel):
                     }
                     signals.append(d)
 
-        signals.sort(key=lambda x: (d["module_name"], d["ns_name"], d["signal_name"]))
+        signals.sort(key=lambda d: (d["module_name"], d["ns_name"], d["signal_name"]))
 
         ctx = {"signals": signals, "events": self.events}
 
@@ -125,11 +118,7 @@ class SignalsDebugPanel(DebugPanel):
         events = self.events
 
         def wrapped_send(self, *sender, **kwargs):
-            d = {
-                "signal_name": self.name,
-                "sender": text_type(sender[0]),
-                "args": kwargs,
-            }
+            d = {"signal_name": self.name, "sender": str(sender[0]), "args": kwargs}
             events.append(d)
             return orig_send(self, *sender, **kwargs)
 
