@@ -1,20 +1,24 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+import typing
+from typing import Dict
 
-from flask import current_app, jsonify, send_file
+from flask import current_app, send_file
 from flask_login import current_user
 from flask_wtf.file import FileField, file_required
 from werkzeug.exceptions import BadRequest, NotFound
 from werkzeug.utils import secure_filename
 
+from abilian.core.models.subjects import User
 from abilian.core.util import unwrap
 from abilian.web import csrf, url_for
 from abilian.web.blueprints import Blueprint
 from abilian.web.forms import Form
 from abilian.web.views import JSONView, View
 
+if typing.TYPE_CHECKING:
+    from abilian.web.uploads import FileUploadsExtension
+
+
 bp = Blueprint("uploads", __name__, url_prefix="/upload")
 
 
@@ -23,9 +27,12 @@ class UploadForm(Form):
     file = FileField(validators=(file_required(),))
 
 
-class BaseUploadsView(object):
+class BaseUploadsView:
+    uploads: "FileUploadsExtension"
+    user: User
+
     def prepare_args(self, args, kwargs):
-        args, kwargs = super(BaseUploadsView, self).prepare_args(args, kwargs)
+        args, kwargs = super().prepare_args(args, kwargs)
         self.uploads = current_app.extensions["uploads"]
         self.user = unwrap(current_user)
         return args, kwargs
@@ -40,7 +47,7 @@ class NewUploadView(BaseUploadsView, JSONView):
     #: file handle to be returned
     handle = None
 
-    def data(self, *args, **kwargs):
+    def data(self, *args, **kwargs) -> Dict:
         return {"handle": self.handle, "url": url_for(".handle", handle=self.handle)}
 
     def post(self, *args, **kwargs):
@@ -90,12 +97,11 @@ class UploadView(BaseUploadsView, View):
             add_etags=False,
         )
 
-    def delete(self, handle, *args, **kwargs):
+    def delete(self, handle, *args, **kwargs) -> Dict:
         if self.uploads.get_file(self.user, handle) is None:
             raise NotFound()
-
         self.uploads.remove_file(self.user, handle)
-        return jsonify({"success": True})
+        return {"success": True}
 
 
 bp.add_url_rule("/<string:handle>", view_func=UploadView.as_view("handle"))
