@@ -1,26 +1,23 @@
-# coding=utf-8
 """A few utility functions.
 
 See https://docs.djangoproject.com/en/dev/topics/http/shortcuts/ for
 more ideas of stuff to implement.
 """
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any
 
 from flask import current_app
 from flask import url_for as flask_url_for
 from flask.helpers import send_from_directory
-from six import string_types
 from werkzeug.routing import BuildError
 
 
-def url_for(obj, **kw):
+def url_for(obj: Any, **kw: Any) -> str:
     """Polymorphic variant of Flask's `url_for` function.
 
     Behaves like the original function when the first argument is a
     string. When it's an object, it
     """
-    if isinstance(obj, string_types):
+    if isinstance(obj, str):
         return flask_url_for(obj, **kw)
 
     try:
