@@ -1,14 +1,11 @@
-# coding=utf-8
 """"""
-from __future__ import absolute_import, division, print_function, \
-    unicode_literals
+from typing import Any, Callable, Dict, Tuple
 
 from flask import g, json, jsonify, redirect, render_template_string, request
 from flask.views import MethodView as BaseView
-from six import PY2
 from werkzeug.exceptions import HTTPException
 
-from ..action import actions
+from abilian.web.action import actions
 
 
 class Redirect(HTTPException):
@@ -23,18 +20,16 @@ class View(BaseView):
     """
 
     @classmethod
-    def as_view(cls, name, *class_args, **class_kwargs):
-        if PY2:
-            name = bytes(name)
-        return super(View, cls).as_view(name, *class_args, **class_kwargs)
+    def as_view(cls, name: str, *class_args: Any, **class_kwargs: Any) -> Callable:
+        return super().as_view(name, *class_args, **class_kwargs)
 
-    def dispatch_request(self, *args, **kwargs):
+    def dispatch_request(self, *args: Any, **kwargs: Any) -> str:
         meth = getattr(self, request.method.lower(), None)
         # if the request method is HEAD and we don't have a handler for it
         # retry with GET
         if meth is None and request.method == "HEAD":
             meth = getattr(self, "get", None)
-            assert meth is not None, "Unimplemented method %r" % request.method
+            assert meth is not None, f"Unimplemented method {request.method!r}"
 
         g.view = actions.context["view"] = self
         try:
@@ -91,15 +86,17 @@ class JSONView(View):
     requested from browser.
     """
 
-    def prepare_args(self, args, kwargs):
+    def prepare_args(
+        self, args: Tuple, kwargs: Dict[Any, Any]
+    ) -> Tuple[Tuple, Dict[Any, Any]]:
         kwargs.update({k: v for k, v in request.args.items()})
         return args, kwargs
 
-    def data(self, *args, **kwargs):
+    def data(self, *args, **kwargs) -> Dict:
         """This method should return data to be serialized using JSON."""
         raise NotImplementedError
 
-    def get(self, *args, **kwargs):
+    def get(self, *args: Any, **kwargs: Any) -> str:
         data = self.data(*args, **kwargs)
         best_mime = request.accept_mimetypes.best_match(
             ["text/html", "application/json"]
